<?php

/*
 * Chill is a software for social workers
 *
 * Copyright (C) 2014-2021, Champs Libres Cooperative SCRLFS,
 * <http://www.champs-libres.coop>, <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\ActivityBundle\DataFixtures\ORM;

use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\Common\DataFixtures\OrderedFixtureInterface;
use Doctrine\Persistence\ObjectManager;
use Chill\ActivityBundle\Entity\ActivityTypeCategory;

/**
 * Fixtures for ActivityTypeCategory
 *
 * @author Champs-Libres Coop
 */
class LoadActivityTypeCategory extends Fixture implements OrderedFixtureInterface
{
    public static $references = array();

    public function getOrder()
    {
        return 16050;
    }

    public function load(ObjectManager $manager)
    {
        $categories = [
            [
                'name' => ['fr' => 'Échange avec usager', 'en' => 'Exchange with user'],
                'ref' => 'exchange',
            ],
            [
                'name' => ['fr' => 'Réunion', 'en' => 'Meeting'],
                'ref' => 'meeting',
            ],
        ];

        foreach ($categories as $cat) {
            print "Creating activity type category : " . $cat['ref'] . "\n";

            $newCat = (new ActivityTypeCategory())
                ->setName(($cat['name']));

            $manager->persist($newCat);
            $reference = 'activity_type_cat_'.$cat['ref'];

            $this->addReference($reference, $newCat);
            static::$references[] = $reference;
        }

        $manager->flush();
    }
}
