<?php

/*
 *
 * Copyright (C) 2015, Champs Libres Cooperative SCRLFS, <http://www.champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\ActivityBundle\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * Class ActivityType
 *
 * @package Chill\ActivityBundle\Entity
 * @ORM\Entity()
 * @ORM\Table(name="activitytype")
 * @ORM\HasLifecycleCallbacks()
 */
class ActivityType
{
    const FIELD_INVISIBLE = 0;
    const FIELD_OPTIONAL = 1;
    const FIELD_REQUIRED = 2;

    /**
     * @ORM\Id
     * @ORM\Column(name="id", type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private ?int $id;

    /**
     * @ORM\Column(type="json")
     */
    private array $name = [];

    /**
     * @ORM\Column(type="boolean")
     */
    private bool $active = true;

    /**
     * @ORM\ManyToOne(targetEntity="Chill\ActivityBundle\Entity\ActivityTypeCategory")
     */
    private ?ActivityTypeCategory $category = null;

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=2})
     */
    private int $personVisible = self::FIELD_REQUIRED;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $personLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=2})
     */
    private int $userVisible = self::FIELD_REQUIRED;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $userLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=2})
     */
    private int $dateVisible = self::FIELD_REQUIRED;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $dateLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $placeVisible = self::FIELD_OPTIONAL;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $placeLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $personsVisible = self::FIELD_OPTIONAL;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $personsLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $thirdPartiesVisible = self::FIELD_INVISIBLE;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $thirdPartiesLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $durationTimeVisible = self::FIELD_OPTIONAL;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $durationTimeLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $travelTimeVisible = self::FIELD_OPTIONAL;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $travelTimeLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $attendeeVisible = self::FIELD_OPTIONAL;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $attendeeLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $reasonsVisible = self::FIELD_OPTIONAL;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $reasonsLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $commentVisible = self::FIELD_OPTIONAL;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $commentLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $sentReceivedVisible = self::FIELD_OPTIONAL;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $sentReceivedLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $documentsVisible = self::FIELD_OPTIONAL;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $documentsLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $usersVisible = self::FIELD_OPTIONAL;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $usersLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $emergencyVisible = self::FIELD_INVISIBLE;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $emergencyLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $accompanyingPeriodVisible = self::FIELD_INVISIBLE;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $accompanyingPeriodLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $socialDataVisible = self::FIELD_INVISIBLE;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $socialDataLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $socialIssuesVisible = self::FIELD_INVISIBLE;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $socialIssuesLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $socialActionsVisible = self::FIELD_INVISIBLE;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $socialActionsLabel = '';

    /**
     * @ORM\Column(type="smallint", nullable=false, options={"default"=1})
     */
    private int $locationVisible = self::FIELD_INVISIBLE;

    /**
     * @ORM\Column(type="string", nullable=false, options={"default"=""})
     */
    private string $locationLabel = '';

    /**
     * @ORM\Column(type="float", options={"default"="0.0"})
     */
    private float $ordering = 0.0;

    /**
     * Get id
     */
    public function getId(): int
    {
        return $this->id;
    }

    /**
     * Set name
     */
    public function setName(array $name): self
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     */
    public function getName(): array
    {
        return $this->name;
    }

    /**
     * Get active
     * return true if the type is active.
     */
    public function getActive(): bool
    {
        return $this->active;
    }

    /**
     * Is active
     * return true if the type is active
     */
    public function isActive(): bool
    {
        return $this->getActive();
    }

    /**
     * Set active
     * set to true if the type is active
     */
    public function setActive(bool $active): self
    {
        $this->active = $active;

        return $this;
    }

    public function getCategory(): ?ActivityTypeCategory
    {
        return $this->category;
    }

    public function setCategory(?ActivityTypeCategory $category): self
    {
        $this->category = $category;

        return $this;
    }

    public function getPersonVisible(): int
    {
        return $this->personVisible;
    }

    public function setPersonVisible(int $personVisible): self
    {
        $this->personVisible = $personVisible;

        return $this;
    }

    public function getPersonLabel(): string
    {
        return $this->personLabel;
    }

    public function setPersonLabel(string $personLabel): self
    {
        $this->personLabel = $personLabel;

        return $this;
    }

    public function getUserVisible(): int
    {
        return $this->userVisible;
    }

    public function setUserVisible(int $userVisible): self
    {
        $this->userVisible = $userVisible;

        return $this;
    }

    public function getUserLabel(): string
    {
        return $this->userLabel;
    }

    public function setUserLabel(string $userLabel): self
    {
        $this->userLabel = $userLabel;

        return $this;
    }

    public function getDateVisible(): int
    {
        return $this->dateVisible;
    }

    public function setDateVisible(int $dateVisible): self
    {
        $this->dateVisible = $dateVisible;

        return $this;
    }

    public function getDateLabel(): string
    {
        return $this->dateLabel;
    }

    public function setDateLabel(string $dateLabel): self
    {
        $this->dateLabel = $dateLabel;

        return $this;
    }

    public function getPlaceVisible(): int
    {
        return $this->placeVisible;
    }

    public function setPlaceVisible(int $placeVisible): self
    {
        $this->placeVisible = $placeVisible;

        return $this;
    }

    public function getPlaceLabel(): string
    {
        return $this->placeLabel;
    }

    public function setPlaceLabel(string $placeLabel): self
    {
        $this->placeLabel = $placeLabel;

        return $this;
    }

    public function getPersonsVisible(): int
    {
        return $this->personsVisible;
    }

    public function setPersonsVisible(int $personsVisible): self
    {
        $this->personsVisible = $personsVisible;

        return $this;
    }

    public function getPersonsLabel(): string
    {
        return $this->personsLabel;
    }

    public function setPersonsLabel(string $personsLabel): self
    {
        $this->personsLabel = $personsLabel;

        return $this;
    }

    public function getThirdPartiesVisible(): int
    {
        return $this->thirdPartiesVisible;
    }

    public function setThirdPartiesVisible(int $thirdPartiesVisible): self
    {
        $this->thirdPartiesVisible = $thirdPartiesVisible;

        return $this;
    }

    public function getThirdPartiesLabel(): string
    {
        return $this->thirdPartiesLabel;
    }

    public function setThirdPartiesLabel(string $thirdPartiesLabel): self
    {
        $this->thirdPartiesLabel = $thirdPartiesLabel;

        return $this;
    }

    public function getDurationTimeVisible(): int
    {
        return $this->durationTimeVisible;
    }

    public function setDurationTimeVisible(int $durationTimeVisible): self
    {
        $this->durationTimeVisible = $durationTimeVisible;

        return $this;
    }

    public function getDurationTimeLabel(): string
    {
        return $this->durationTimeLabel;
    }

    public function setDurationTimeLabel(string $durationTimeLabel): self
    {
        $this->durationTimeLabel = $durationTimeLabel;

        return $this;
    }

    public function getTravelTimeVisible(): int
    {
        return $this->travelTimeVisible;
    }

    public function setTravelTimeVisible(int $TravelTimeVisible): self
    {
        $this->travelTimeVisible = $TravelTimeVisible;

        return $this;
    }

    public function getTravelTimeLabel(): string
    {
        return $this->travelTimeLabel;
    }

    public function setTravelTimeLabel(string $TravelTimeLabel): self
    {
        $this->travelTimeLabel = $TravelTimeLabel;

        return $this;
    }

    public function getAttendeeVisible(): int
    {
        return $this->attendeeVisible;
    }

    public function setAttendeeVisible(int $attendeeVisible): self
    {
        $this->attendeeVisible = $attendeeVisible;

        return $this;
    }

    public function getAttendeeLabel(): string
    {
        return $this->attendeeLabel;
    }

    public function setAttendeeLabel(string $attendeeLabel): self
    {
        $this->attendeeLabel = $attendeeLabel;

        return $this;
    }

    public function getReasonsVisible(): int
    {
        return $this->reasonsVisible;
    }

    public function setReasonsVisible(int $reasonsVisible): self
    {
        $this->reasonsVisible = $reasonsVisible;

        return $this;
    }

    public function getReasonsLabel(): string
    {
        return $this->reasonsLabel;
    }

    public function setReasonsLabel(string $reasonsLabel): self
    {
        $this->reasonsLabel = $reasonsLabel;

        return $this;
    }

    public function getCommentVisible(): int
    {
        return $this->commentVisible;
    }

    public function setCommentVisible(int $commentVisible): self
    {
        $this->commentVisible = $commentVisible;

        return $this;
    }

    public function getCommentLabel(): string
    {
        return $this->commentLabel;
    }

    public function setCommentLabel(string $commentLabel): self
    {
        $this->commentLabel = $commentLabel;

        return $this;
    }

    public function getSentReceivedVisible(): int
    {
        return $this->sentReceivedVisible;
    }

    public function setSentReceivedVisible(int $sentReceivedVisible): self
    {
        $this->sentReceivedVisible = $sentReceivedVisible;

        return $this;
    }

    public function getSentReceivedLabel(): string
    {
        return $this->sentReceivedLabel;
    }

    public function setSentReceivedLabel(string $sentReceivedLabel): self
    {
        $this->sentReceivedLabel = $sentReceivedLabel;

        return $this;
    }

    public function getDocumentsVisible(): int
    {
        return $this->documentsVisible;
    }

    public function setDocumentsVisible(int $documentsVisible): self
    {
        $this->documentsVisible = $documentsVisible;

        return $this;
    }

    public function getDocumentsLabel(): string
    {
        return $this->documentsLabel;
    }

    public function setDocumentsLabel(string $documentsLabel): self
    {
        $this->documentsLabel = $documentsLabel;

        return $this;
    }

    public function getUsersVisible(): int
    {
        return $this->usersVisible;
    }

    public function setUsersVisible(int $usersVisible): self
    {
        $this->usersVisible = $usersVisible;

        return $this;
    }

    public function getUsersLabel(): string
    {
        return $this->usersLabel;
    }

    public function setUsersLabel(string $usersLabel): self
    {
        $this->usersLabel = $usersLabel;

        return $this;
    }

    public function getEmergencyVisible(): int
    {
        return $this->emergencyVisible;
    }

    public function setEmergencyVisible(int $emergencyVisible): self
    {
        $this->emergencyVisible = $emergencyVisible;

        return $this;
    }

    public function getEmergencyLabel(): string
    {
        return $this->emergencyLabel;
    }

    public function setEmergencyLabel(string $emergencyLabel): self
    {
        $this->emergencyLabel = $emergencyLabel;

        return $this;
    }

    public function getAccompanyingPeriodVisible(): int
    {
        return $this->accompanyingPeriodVisible;
    }

    public function setAccompanyingPeriodVisible(int $accompanyingPeriodVisible): self
    {
        $this->accompanyingPeriodVisible = $accompanyingPeriodVisible;

        return $this;
    }

    public function getAccompanyingPeriodLabel(): string
    {
        return $this->accompanyingPeriodLabel;
    }

    public function setAccompanyingPeriodLabel(string $accompanyingPeriodLabel): self
    {
        $this->accompanyingPeriodLabel = $accompanyingPeriodLabel;

        return $this;
    }

    public function getSocialDataVisible(): int
    {
        return $this->socialDataVisible;
    }

    public function setSocialDataVisible(int $socialDataVisible): self
    {
        $this->socialDataVisible = $socialDataVisible;

        return $this;
    }

    public function getSocialDataLabel(): string
    {
        return $this->socialDataLabel;
    }

    public function setSocialDataLabel(string $socialDataLabel): self
    {
        $this->socialDataLabel = $socialDataLabel;

        return $this;
    }

    public function isVisible(string $field): bool
    {
        $property = $field.'Visible';

        if (!property_exists($this, $property)) {
            throw new \InvalidArgumentException('Field "'.$field.'" not found');
        }

        return self::FIELD_INVISIBLE !== $this->$property;
    }

    public function isRequired(string $field): bool
    {
        $property = $field.'Visible';

        if (!property_exists($this, $property)) {
            throw new \InvalidArgumentException('Field "'.$field.'" not found');
        }

        return self::FIELD_REQUIRED === $this->$property;
    }

    public function getLabel(string $field): ?string
    {
        $property = $field.'Label';

        if (!property_exists($this, $property)) {
            throw new \InvalidArgumentException('Field "'.$field.'" not found');
        }

        return $this->$property;
    }

    public function getOrdering(): float
    {
        return $this->ordering;
    }

    public function setOrdering(float $ordering): self
    {
        $this->ordering = $ordering;

        return $this;
    }

    public function getSocialIssuesVisible(): ?int
    {
        return $this->socialIssuesVisible;
    }

    public function setSocialIssuesVisible(int $socialIssuesVisible): self
    {
        $this->socialIssuesVisible = $socialIssuesVisible;

        return $this;
    }

    public function getSocialIssuesLabel(): ?string
    {
        return $this->socialIssuesLabel;
    }

    public function setSocialIssuesLabel(string $socialIssuesLabel): self
    {
        $this->socialIssuesLabel = $socialIssuesLabel;

        return $this;
    }

    public function getSocialActionsVisible(): ?int
    {
        return $this->socialActionsVisible;
    }

    public function setSocialActionsVisible(int $socialActionsVisible): self
    {
        $this->socialActionsVisible = $socialActionsVisible;

        return $this;
    }

    public function getSocialActionsLabel(): ?string
    {
        return $this->socialActionsLabel;
    }

    public function setSocialActionsLabel(string $socialActionsLabel): self
    {
        $this->socialActionsLabel = $socialActionsLabel;

        return $this;
    }

    public function getLocationVisible(): ?int
    {
        return $this->locationVisible;
    }

    public function setLocationVisible(int $locationVisible): self
    {
        $this->locationVisible = $locationVisible;

        return $this;
    }

    public function getLocationLabel(): ?string
    {
        return $this->locationLabel;
    }

    public function setLocationLabel(string $locationLabel): self
    {
        $this->locationLabel = $locationLabel;

        return $this;
    }

}
