<?php

declare(strict_types=1);

namespace Chill\ActivityBundle\Export\Aggregator;

use Chill\ActivityBundle\Repository\ActivityTypeRepository;
use Chill\MainBundle\Templating\TranslatableStringHelperInterface;
use Symfony\Component\Form\FormBuilderInterface;
use Doctrine\ORM\QueryBuilder;
use Chill\MainBundle\Export\AggregatorInterface;
use Symfony\Component\Security\Core\Role\Role;
use Chill\ActivityBundle\Security\Authorization\ActivityStatsVoter;
use Doctrine\ORM\Query\Expr\Join;

class ActivityTypeAggregator implements AggregatorInterface
{
    protected ActivityTypeRepository $activityTypeRepository;

    protected TranslatableStringHelperInterface $translatableStringHelper;

    public const KEY = 'activity_type_aggregator';

    public function __construct(
        ActivityTypeRepository $activityTypeRepository,
        TranslatableStringHelperInterface $translatableStringHelper
    ) {
        $this->activityTypeRepository = $activityTypeRepository;
        $this->translatableStringHelper = $translatableStringHelper;
    }

    public function alterQuery(QueryBuilder $qb, $data)
    {
        // add select element
        $qb->addSelect(sprintf('IDENTITY(activity.type) AS %s', self::KEY));

        // add the "group by" part
        $qb->addGroupBy(self::KEY);
    }

    /**
     * Check if a join between Activity and another alias
     *
     * @param Join[] $joins
     * @param string $alias the alias to search for
     * @return boolean
     */
    private function checkJoinAlreadyDefined(array $joins, $alias)
    {
        foreach ($joins as $join) {
            if ($join->getAlias() === $alias) {
                return true;
            }
        }

        return false;
    }

    public function applyOn()
    {
        return 'activity';
    }

    public function buildForm(FormBuilderInterface $builder)
    {
        // no form required for this aggregator
    }

    public function getTitle()
    {
        return 'Aggregate by activity type';
    }

    public function addRole()
    {
        return new Role(ActivityStatsVoter::STATS);
    }

    public function getLabels($key, array $values, $data): \Closure
    {
        // for performance reason, we load data from db only once
        $this->activityTypeRepository->findBy(['id' => $values]);

        return function($value): string {
            if ($value === '_header') {
                return 'Activity type';
            }

            $t = $this->activityTypeRepository->find($value);

            return $this->translatableStringHelper->localize($t->getName());
        };
    }

    public function getQueryKeys($data): array
    {
        return [self::KEY];
    }

}
