<?php

declare(strict_types=1);

namespace Chill\ActivityBundle\Form\Type;

use Chill\ActivityBundle\Repository\ActivityTypeRepository;
use Chill\MainBundle\Templating\TranslatableStringHelperInterface;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\QueryBuilder;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Chill\ActivityBundle\Entity\ActivityType;

class TranslatableActivityType extends AbstractType
{
    protected TranslatableStringHelperInterface $translatableStringHelper;

    protected ActivityTypeRepository $activityTypeRepository;

    public function __construct(
        TranslatableStringHelperInterface $helper,
        ActivityTypeRepository $activityTypeRepository
    ) {
        $this->translatableStringHelper = $helper;
        $this->activityTypeRepository = $activityTypeRepository;
    }

    public function getBlockPrefix()
    {
        return 'translatable_activity_type';
    }

    public function getParent()
    {
        return EntityType::class;
    }

    public function buildForm(FormBuilderInterface $builder, array $options) {
        /* @var QueryBuilder $qb */
        $qb = $options['query_builder'];

        if ($options['active_only'] === true) {
            $qb->where($qb->expr()->eq('at.active', ':active'));
            $qb->setParameter('active', true, Types::BOOLEAN);
        }
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(
            array(
                'class' => ActivityType::class,
                'active_only' => true,
                'query_builder' => $this->activityTypeRepository
                    ->createQueryBuilder('at'),
                'choice_label' => function (ActivityType $type) {
                    return $this->translatableStringHelper->localize($type->getName());
                }
            )
        );
    }
}
