<?php

namespace Chill\ActivityBundle\Menu;

use Chill\ActivityBundle\Security\Authorization\ActivityVoter;
use Chill\MainBundle\Routing\LocalMenuBuilderInterface;
use Chill\MainBundle\Security\Authorization\AuthorizationHelper;
use Chill\PersonBundle\Entity\AccompanyingPeriod;
use Knp\Menu\MenuItem;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Security\Core\Security;
use Symfony\Contracts\Translation\TranslatorInterface;

class AccompanyingCourseMenuBuilder implements LocalMenuBuilderInterface
{
    protected TranslatorInterface $translator;

    protected Security $security;

    public function __construct(
        Security $security,
        TranslatorInterface $translator
    ) {
        $this->security = $security;
        $this->translator = $translator;
    }
    public static function getMenuIds(): array
    {
        return ['accompanyingCourse'];
    }

    public function buildMenu($menuId, MenuItem $menu, array $parameters)
    {
        $period = $parameters['accompanyingCourse'];

        if (AccompanyingPeriod::STEP_DRAFT !== $period->getStep()
            && $this->security->isGranted(ActivityVoter::SEE, $period)) {
            $menu->addChild($this->translator->trans('Activity'), [
                'route' => 'chill_activity_activity_list',
                'routeParameters' => [
                    'accompanying_period_id' => $period->getId(),
                ]])
                ->setExtras(['order' => 40]);
        }
    }
}
