<?php
/*
 * Copyright (C) 2018 Julien Fastré <julien.fastre@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
namespace Chill\ActivityBundle\Menu; 
use Chill\MainBundle\Routing\LocalMenuBuilderInterface;
use Knp\Menu\MenuItem;
use Symfony\Component\Security\Core\Authorization\AuthorizationCheckerInterface;
use Chill\ActivityBundle\Security\Authorization\ActivityVoter;
use Symfony\Component\Translation\TranslatorInterface;

/**
 * 
 *
 * @author Julien Fastré <julien.fastre@champs-libres.coop>
 */
class PersonMenuBuilder implements LocalMenuBuilderInterface
{
    /**
     *
     * @var TranslatorInterface
     */
    protected $translator;
    
    /**
     *
     * @var AuthorizationCheckerInterface
     */
    protected $authorizationChecker;
    
    public function __construct(
        AuthorizationCheckerInterface $authorizationChecker,
        TranslatorInterface $translator)
    {
        $this->translator = $translator;
        $this->authorizationChecker = $authorizationChecker;
    }

    
    public function buildMenu($menuId, MenuItem $menu, array $parameters)
    {
        /* @var $person \Chill\PersonBundle\Entity\Person */
        $person = $parameters['person'];
        
        if ($this->authorizationChecker->isGranted(ActivityVoter::SEE, $person)) {
            $menu->addChild(
                $this->translator->trans('Activity list'), [
                    'route' => 'chill_activity_activity_list',
                    'routeParameters' => [ 'person_id' => $person->getId() ],
                ])
                ->setExtra('order', 201)
                ;
        }
    }

    public static function getMenuIds(): array
    {
        return ['person'];
    }
}
