<?php

/*
 * Copyright (C) 2016 Champs-Libres <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\ActivityBundle\Security\Authorization;

use Chill\MainBundle\Security\Authorization\AbstractChillVoter;
use Chill\MainBundle\Security\Authorization\AuthorizationHelper;
use Chill\MainBundle\Security\ProvideRoleHierarchyInterface;
use Chill\MainBundle\Entity\Center;

/**
 * 
 *
 * @author Julien Fastré <julien.fastre@champs-libres.coop>
 */
class ActivityStatsVoter extends AbstractChillVoter implements ProvideRoleHierarchyInterface
{
    const STATS = 'CHILL_ACTIVITY_STATS';
    const LISTS  = 'CHILL_ACTIVITY_LIST';
    
    /**
     *
     * @var AuthorizationHelper
     */
    protected $helper;
    
    public function __construct(AuthorizationHelper $helper)
    {
        $this->helper = $helper;
    }

    private function getAttributes()
    {
        return array(self::STATS, self::LISTS);
    }

    protected function getSupportedClasses()
    {
        return array(Center::class);
    }
    
    protected function supports($attribute, $subject)
    {
        if ($subject instanceof Center 
            && \in_array($attribute, $this->getAttributes())) {
            
            return true;
        }
        
        return false;
    }

    protected function isGranted($attribute, $object, $user = null)
    {
        if (!$user instanceof \Symfony\Component\Security\Core\User\UserInterface) {
            return false;
        }
        
        return $this->helper->userHasAccess($user, $object, $attribute);
    }

    public function getRoles()
    {
        return $this->getAttributes();
    }

    public function getRolesWithoutScope()
    {
        return $this->getAttributes();
    }

    public function getRolesWithHierarchy()
    {
        return [ 'Activity' => $this->getRoles() ];
    }
}
