<?php
/*
 * Chill is a software for social workers
 *
 * Copyright (C) 2014-2020, Champs Libres Cooperative SCRLFS,
 * <http://www.champs-libres.coop>, <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
namespace Chill\ActivityBundle\Templating\Entity;

use Chill\MainBundle\Templating\Entity\AbstractChillEntityRender;
use Chill\ActivityBundle\Entity\ActivityReason;
use Chill\MainBundle\Templating\TranslatableStringHelper;

/**
 * Render activity reason
 *
 */
class ActivityReasonRender extends AbstractChillEntityRender
{
    /**
     *
     * @var TranslatableStringHelper
     */
    protected $translatableStringHelper;

    public function __construct(TranslatableStringHelper $translatableStringHelper)
    {
        $this->translatableStringHelper = $translatableStringHelper;
    }


    public function renderBox($entity, array $options): string
    {
        return
            $this->getDefaultOpeningBox('activity-reason').
            '<span class="badge bg-chill-pink">'.
            '<i class="fa fa-question-circle"></i>&nbsp;'.
            '<span class="category">'.
            $this->translatableStringHelper->localize(
                $entity->getCategory()->getName()
            ).
            '</span>'.
            '<span class="separator">&nbsp;>&nbsp;</span>'.
            '<span class="reason">'.
            $this->translatableStringHelper->localize(
                $entity->getName()
            ).
            '</span>'.
            '</span>'.
            $this->getDefaultClosingBox()
            ;
    }

    /**
     *
     * @param ActivityReason $entity
     * @param array $options
     * @return string
     */
    public function renderString($entity, array $options): string
    {
        $category = '';

        if (null !== $entity->getCategory()) {
            $category = $this->translatableStringHelper->localize(
                $entity->getCategory()->getName()). ' > ';
        }

        return $category .
            $this->translatableStringHelper->localize(
                $entity->getName()
            );
    }

    public function supports($entity, array $options): bool
    {
        return $entity instanceof ActivityReason;
    }
}
