<?php
/*
 * Copyright (C) 2017 Champs Libres Cooperative <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
namespace Chill\ActivityBundle\Tests\Filter;

use Chill\MainBundle\Test\Export\AbstractFilterTest;

/**
 * 
 *
 * @author Julien Fastré <julien.fastre@champs-libres.coop>
 */
class PersonHavingActivityBetweenDateFilterTest extends AbstractFilterTest
{
    /**
     *
     * @var \Chill\PersonBundle\Export\Filter\PersonHavingActivityBetweenDateFilter
     */
    private $filter;
    
    public function setUp()
    {
        static::bootKernel();
        
        $container = static::$kernel->getContainer();
        
        $this->filter = $container->get('chill.activity.export.'
            . 'person_having_an_activity_between_date_filter');
        
        // add a fake request with a default locale (used in translatable string)
        $prophet = new \Prophecy\Prophet;
        $request = $prophet->prophesize();
        $request->willExtend(\Symfony\Component\HttpFoundation\Request::class);
        $request->getLocale()->willReturn('fr');
        
        $container->get('request_stack')
                ->push($request->reveal());
    }
    
    public function getFilter()
    {
        return $this->filter;
    }

    public function getFormData()
    {
        $date_from = \DateTime::createFromFormat('Y-m-d', '2015-01-15');
        $date_to   = new \DateTime(); // today
        $reasons = $this->getActivityReasons();

        
        $data = array();
        for ($i = 0; $i < 4; $i++) {
            $data[] = array(
                'date_from' => $date_from,
                'date_to'   => $date_to,
                'reasons'   => array_slice($reasons, 0, 1 + $i)
            );
        }
        
        return $data;
    }
    
    /**
     * Return all activity reasons
     * 
     * @return \Chill\ActivityBundle\Entity\ActivityReason[]
     */
    private function getActivityReasons() 
    {
        if (static::$kernel === null) {
            static::bootKernel();
        }
        
        return static::$kernel->getContainer()
            ->get('chill_activity.repository.reason')
            ->findAll();
    }

    public function getQueryBuilders()
    {
        if (static::$kernel === null) {
            static::bootKernel();
        }
        
        $em = static::$kernel->getContainer()
            ->get('doctrine.orm.entity_manager');
        
        return array(
            $em->createQueryBuilder()
                ->select('count(person.id)')
                ->from('ChillPersonBundle:Person', 'person')
                // add a fake where clause
                ->where('person.id > 0'),
            $em->createQueryBuilder()
                ->select('count(person.id)')
                ->from('ChillActivityBundle:Activity', 'activity')
                ->join('activity.person', 'person')
                // add a fake where clause
                ->where('person.id > 0'),
        );
    }
}
