<?php

namespace Chill\AsideActivityBundle\Menu;

use Chill\MainBundle\Routing\LocalMenuBuilderInterface;
use Knp\Menu\MenuItem;
use Symfony\Component\Security\Core\Authorization\AuthorizationChecker;
use Symfony\Component\Security\Core\Authorization\AuthorizationCheckerInterface;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Class SectionMenuBuilder
 *
 * @package Chill\AsideActivityBundle\Menu
 */
class SectionMenuBuilder implements LocalMenuBuilderInterface
{
    protected TranslatorInterface $translator;
    public AuthorizationCheckerInterface $authorizationChecker;

    public function __construct(TranslatorInterface $translator, AuthorizationCheckerInterface $authorizationChecker)
    {
        $this->translator = $translator;
        $this->authorizationChecker = $authorizationChecker;
    }

    /**
     * @param $menuId
     * @param MenuItem $menu
     * @param array $parameters
     */
    public function buildMenu($menuId, MenuItem $menu, array $parameters)
    {
        if ($this->authorizationChecker->isGranted('ROLE_USER')){
            $menu->addChild($this->translator->trans('Create an aside activity'), [
                    'route' => 'chill_crud_aside_activity_new'
                ])
                ->setExtras([
                    'order' => 11,
                    'icons' => [ 'plus' ]
                ]);
            $menu->addChild($this->translator->trans('Phonecall'), [
                'route' => 'chill_crud_aside_activity_new',
                'routeParameters' => [
                    'type' => 1,
                    'duration' => 900,
                ]
            ])
            ->setExtras([
                'order' => 12,
                'icons' => ['plus']
            ]);
        }
    }

    /**
     * @return array
     */
    public static function getMenuIds(): array
    {
        return [ 'section' ];
    }
}
