<?php

namespace Chill\AsideActivityBundle\Templating\Entity;

use Chill\AsideActivityBundle\Entity\AsideActivityCategory;
use Chill\MainBundle\Templating\Entity\ChillEntityRenderInterface;
use Chill\MainBundle\Templating\TranslatableStringHelper;
use Symfony\Component\Templating\EngineInterface;

final class CategoryRender implements ChillEntityRenderInterface
{
    private TranslatableStringHelper $translatableStringHelper;
    private EngineInterface $engine;

    public const SEPERATOR_KEY = 'default.separator';
    public const DEFAULT_ARGS = [
        self::SEPERATOR_KEY => ' > '
    ];

    public function __construct(TranslatableStringHelper $translatableStringHelper, EngineInterface $engine)
    {
        $this->translatableStringHelper = $translatableStringHelper;
        $this->engine = $engine;
    }

    /**
     * @param AsideActivityCategory $asideActivityCategory
     */
    public function renderString($asideActivityCategory, array $options): string
    {
        $options = array_merge(self::DEFAULT_ARGS, $options);

        $titles = [
            $this->translatableStringHelper->localize($asideActivityCategory->getTitle()),
        ];

        while ($asideActivityCategory->hasParent()) {
            $asideActivityCategory = $asideActivityCategory->getParent();
            $titles[] = $this->translatableStringHelper->localize($asideActivityCategory->getTitle());
        }

        $titles = array_reverse($titles);

        return implode($options[self::SEPERATOR_KEY], $titles);

    }

    /**
     * @param AsideActivityCategory $asideActivityCategory
     */
    public function supports($asideActivityCategory, array $options): bool
    {
        return $asideActivityCategory instanceof AsideActivityCategory;
    }

    public function buildParents(AsideActivityCategory $asideActivityCategory)
    {
        $parents = [];

        while($asideActivityCategory->hasParent()) {
            $asideActivityCategory = $parents[] = $asideActivityCategory->getParent();
        }

        return $parents;
    }

    /**
     * @param AsideActivityCategory $asideActivityCategory
     */
    public function renderBox($asideActivityCategory, array $options): string
    {
        $options = array_merge(self::DEFAULT_ARGS, $options);
        $parents = $this->buildParents($asideActivityCategory);

        return $this->engine->render('@ChillAsideActivity/Entity/asideActivityCategory.html.twig',
        [
            'asideActivityCategory' => $asideActivityCategory,
            'parents' => $parents,
            'options' => $options
        ]);
    }
}
