<?php

namespace Chill\DocGeneratorBundle\DependencyInjection;

use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\HttpKernel\DependencyInjection\Extension;
use Symfony\Component\DependencyInjection\Loader;
use Symfony\Component\DependencyInjection\Extension\PrependExtensionInterface;

/**
 * This is the class that loads and manages your bundle configuration.
 *
 * @link http://symfony.com/doc/current/cookbook/bundles/extension.html
 */
class ChillDocGeneratorExtension extends Extension implements PrependExtensionInterface
{
    /**
     * {@inheritdoc}
     */
    public function load(array $configs, ContainerBuilder $container)
    {
        $loader = new Loader\YamlFileLoader($container, new FileLocator(__DIR__.'/../config'));
        $loader->load('services.yaml');
        $loader->load('services/controller.yaml');
        $loader->load('services/fixtures.yaml');
        $loader->load('services/form.yaml');
    }

    public function prepend(ContainerBuilder $container)
    {
        $this->preprendRoutes($container);
        $this->prependCruds($container);
    }

    protected function preprendRoutes(ContainerBuilder $container)
    {
        $container->prependExtensionConfig('chill_main', array(
            'routing' => array(
                'resources' => array(
                    '@ChillDocGeneratorBundle/config/routes.yaml'
                )
            )
        ));
    }

    /**
     * @param ContainerBuilder $container
     */
    protected function prependCruds(ContainerBuilder $container)
    {
        $container->prependExtensionConfig('chill_main', [
            'cruds' => [
                [
                    'class' => \Chill\DocGeneratorBundle\Entity\DocGeneratorTemplate::class,
                    'name' => 'docgen_template',
                    'base_path' => '/admin/docgen/template',
                    'form_class' => \Chill\DocGeneratorBundle\Form\DocGeneratorTemplateType::class,
                    'controller' => \Chill\DocGeneratorBundle\Controller\AdminDocGeneratorTemplateController::class,
                    'actions' => [
                        'index' => [
                            'template' => '@ChillDocGenerator/Admin/DocGeneratorTemplate/index.html.twig',
                            'role' => 'ROLE_ADMIN'
                        ],
                        'new'   => [
                            'role' => 'ROLE_ADMIN',
                            'template' => '@ChillDocGenerator/Admin/DocGeneratorTemplate/new.html.twig',
                        ],
                        'edit'  => [
                            'role' => 'ROLE_ADMIN',
                            'template' => '@ChillDocGenerator/Admin/DocGeneratorTemplate/edit.html.twig',
                        ]
                    ]
                ]
            ]
        ]);
    }
}
