<?php

namespace Chill\DocGeneratorBundle\tests\Serializer\Normalizer;

use Chill\MainBundle\Entity\Center;
use Chill\MainBundle\Entity\User;
use Symfony\Bundle\FrameworkBundle\Test\KernelTestCase;
use Symfony\Component\Serializer\Normalizer\AbstractNormalizer;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;

class DocGenObjectNormalizerTest extends KernelTestCase
{
    private NormalizerInterface $normalizer;


    protected function setUp()
    {
        parent::setUp();
        self::bootKernel();

        $this->normalizer = self::$container->get(NormalizerInterface::class);
    }

    public function testNormalizationBasic()
    {
        $user = new User();
        $user->setUsername('User Test');
        $user->setMainCenter($center = new Center());
        $center->setName('test');

        $normalized = $this->normalizer->normalize($user, 'docgen', [ AbstractNormalizer::GROUPS => ['docgen:read']]);
        $expected = [
            'label' => 'User Test',
            'email' => '',
            'mainCenter' => [
                'name' => 'test'
            ]
        ];

        $this->assertEquals($expected, $normalized, "test normalization fo an user");
    }

    public function testNormalizeWithNullValueEmbedded()
    {
        $user = new User();
        $user->setUsername('User Test');

        $normalized = $this->normalizer->normalize($user, 'docgen', [ AbstractNormalizer::GROUPS => ['docgen:read']]);
        $expected = [
            'label' => 'User Test',
            'email' => '',
            'mainCenter' => [
                'name' => ''
            ]
        ];

        $this->assertEquals($expected, $normalized, "test normalization fo an user with null center");
    }

    public function testNormalizeNullObjectWithObjectEmbedded()
    {
        $normalized = $this->normalizer->normalize(null, 'docgen', [
            AbstractNormalizer::GROUPS => ['docgen:read'],
            'docgen:expects' => User::class,
        ]);

        $expected = [
            'label' => '',
            'email' => '',
            'mainCenter' => [
                'name' => ''
            ]
        ];

        $this->assertEquals($expected, $normalized, "test normalization for a null user");

    }


}
