<?php

namespace Chill\DocStoreBundle\Security\Authorization;

use Chill\DocStoreBundle\Entity\AccompanyingCourseDocument;
use Chill\DocStoreBundle\Entity\PersonDocument;
use Chill\MainBundle\Security\Authorization\AbstractChillVoter;
use Chill\MainBundle\Security\Authorization\AuthorizationHelper;
use Chill\MainBundle\Security\Authorization\VoterHelperFactoryInterface;
use Chill\MainBundle\Security\Authorization\VoterHelperInterface;
use Chill\MainBundle\Security\ProvideRoleHierarchyInterface;
use Chill\MainBundle\Entity\User;
use Chill\PersonBundle\Entity\AccompanyingPeriod;
use Chill\PersonBundle\Entity\Person;
use Chill\PersonBundle\Security\Authorization\AccompanyingPeriodVoter;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authorization\AccessDecisionManagerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\Security\Core\Security;

/**
 *
 */
class AccompanyingCourseDocumentVoter extends AbstractChillVoter implements ProvideRoleHierarchyInterface
{
    const CREATE = 'CHILL_ACCOMPANYING_COURSE_DOCUMENT_CREATE';
    const SEE    = 'CHILL_ACCOMPANYING_COURSE_DOCUMENT_SEE';
    const SEE_DETAILS = 'CHILL_ACCOMPANYING_COURSE_DOCUMENT_SEE_DETAILS';
    const UPDATE = 'CHILL_ACCOMPANYING_COURSE_DOCUMENT_UPDATE';
    const DELETE = 'CHILL_ACCOMPANYING_COURSE_DOCUMENT_DELETE';

    protected LoggerInterface $logger;
    protected VoterHelperInterface $voterHelper;
    protected Security $security;

    public function __construct(
        LoggerInterface $logger,
        Security $security,
        VoterHelperFactoryInterface $voterHelperFactory
    ) {
        $this->logger = $logger;
        $this->security = $security;
        $this->voterHelper = $voterHelperFactory
            ->generate(self::class)
            ->addCheckFor(AccompanyingCourseDocument::class, $this->getRoles())
            ->addCheckFor(AccompanyingPeriod::class, [self::SEE, self::CREATE])
            ->build();
    }

    public function getRoles()
    {
        return [
            self::CREATE,
            self::SEE,
            self::SEE_DETAILS,
            self::UPDATE,
            self::DELETE
        ];
    }

    protected function supports($attribute, $subject)
    {
        return $this->voterHelper->supports($attribute, $subject);
    }

    protected function voteOnAttribute($attribute, $subject, TokenInterface $token)
    {
        $this->logger->debug(sprintf("Voting from %s class", self::class));

        if (!$token->getUser() instanceof User) {
            return false;
        }

        if ($subject instanceof AccompanyingCourseDocument
            && !$this->security->isGranted(AccompanyingPeriodVoter::SEE, $subject->getCourse())) {
            return false;
        }

        return $this->voterHelper->voteOnAttribute($attribute, $subject, $token);
    }

    public function getRolesWithoutScope()
    {
        return array();
    }

    public function getRolesWithHierarchy()
    {
        return ['accompanyingCourseDocument' => $this->getRoles() ];
    }
}
