<?php

namespace Chill\EventBundle\Menu;

use Symfony\Component\Security\Core\Authorization\AuthorizationCheckerInterface;
use Symfony\Component\Translation\TranslatorInterface;
use Knp\Menu\MenuItem;
use Chill\MainBundle\Routing\LocalMenuBuilderInterface;
use Chill\EventBundle\Security\Authorization\EventVoter;

class PersonMenuBuilder implements LocalMenuBuilderInterface
{
    
    /**
     *
     * @var TranslatorInterface
     */
    protected $translator;
    
    /**
     *
     * @var AuthorizationCheckerInterface
     */
    protected $authorizationChecker;
    
    public function __construct(
        AuthorizationCheckerInterface $authorizationChecker,
        TranslatorInterface $translator
    ) {
        $this->authorizationChecker = $authorizationChecker;
        $this->translator = $translator;
    }
    
    
    public function buildMenu($menuId, MenuItem $menu, array $parameters)
    {
        /* @var $person \Chill\PersonBundle\Entity\Person */
        $person  = $parameters['person'] ?? null;
        
        if ($this->authorizationChecker->isGranted(EventVoter::SEE, $person)) {
            
            $menu->addChild($this->translator->trans('Events participation'), [
                'route' => 'chill_event__list_by_person',
                'routeParameters' => [
                    'person_id' => $person->getId()
                ]
            ])
            ->setExtras([
                'order' => 500
            ]);
        }
    }
    
    public static function getMenuIds(): array
    {
        return [ 'person' ];
    }
}