<?php

namespace Chill\MainBundle;

use Chill\MainBundle\Routing\LocalMenuBuilderInterface;
use Chill\MainBundle\Search\SearchApiInterface;
use Chill\MainBundle\Search\SearchInterface;
use Chill\MainBundle\Security\Authorization\ChillVoterInterface;
use Chill\MainBundle\Security\ProvideRoleInterface;
use Chill\MainBundle\Security\Resolver\CenterResolverInterface;
use Chill\MainBundle\Security\Resolver\ScopeResolverInterface;
use Chill\MainBundle\Templating\Entity\ChillEntityRenderInterface;
use Symfony\Component\HttpKernel\Bundle\Bundle;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Chill\MainBundle\DependencyInjection\CompilerPass\SearchableServicesCompilerPass;
use Chill\MainBundle\DependencyInjection\ConfigConsistencyCompilerPass;
use Chill\MainBundle\DependencyInjection\CompilerPass\TimelineCompilerClass;
use Chill\MainBundle\DependencyInjection\RoleProvidersCompilerPass;
use Chill\MainBundle\DependencyInjection\CompilerPass\ExportsCompilerPass;
use Chill\MainBundle\DependencyInjection\CompilerPass\WidgetsCompilerPass;
use Chill\MainBundle\DependencyInjection\CompilerPass\NotificationCounterCompilerPass;
use Chill\MainBundle\DependencyInjection\CompilerPass\MenuCompilerPass;
use Chill\MainBundle\DependencyInjection\CompilerPass\ACLFlagsCompilerPass;
use Chill\MainBundle\DependencyInjection\CompilerPass\GroupingCenterCompilerPass;
use Chill\MainBundle\CRUD\CompilerPass\CRUDControllerCompilerPass;
use Chill\MainBundle\Templating\Entity\CompilerPass as RenderEntityCompilerPass;


class ChillMainBundle extends Bundle
{
    public function build(ContainerBuilder $container)
    {
        parent::build($container);

        $container->registerForAutoconfiguration(LocalMenuBuilderInterface::class)
            ->addTag('chill.menu_builder');
        $container->registerForAutoconfiguration(ProvideRoleInterface::class)
            ->addTag('chill.role');
        $container->registerForAutoconfiguration(CenterResolverInterface::class)
            ->addTag('chill_main.center_resolver');
        $container->registerForAutoconfiguration(ScopeResolverInterface::class)
            ->addTag('chill_main.scope_resolver');
        $container->registerForAutoconfiguration(ChillEntityRenderInterface::class)
            ->addTag('chill.render_entity');
        $container->registerForAutoconfiguration(SearchApiInterface::class)
            ->addTag('chill.search_api_provider');

        $container->addCompilerPass(new SearchableServicesCompilerPass());
        $container->addCompilerPass(new ConfigConsistencyCompilerPass());
        $container->addCompilerPass(new TimelineCompilerClass());
        $container->addCompilerPass(new RoleProvidersCompilerPass());
        $container->addCompilerPass(new ExportsCompilerPass());
        $container->addCompilerPass(new WidgetsCompilerPass());
        $container->addCompilerPass(new NotificationCounterCompilerPass());
        $container->addCompilerPass(new MenuCompilerPass());
        $container->addCompilerPass(new ACLFlagsCompilerPass());
        $container->addCompilerPass(new GroupingCenterCompilerPass());
        $container->addCompilerPass(new RenderEntityCompilerPass());
        $container->addCompilerPass(new CRUDControllerCompilerPass());
    }
}
