<?php

/*
 * Copyright (C) 2015 Champs-Libres Coopérative <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\MainBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;
use Chill\MainBundle\Timeline\TimelineBuilder;
use Chill\MainBundle\Pagination\PaginatorFactory;
use Chill\PersonBundle\Security\Authorization\PersonVoter;
use Chill\MainBundle\Security\Authorization\AuthorizationHelper;
use Symfony\Component\Security\Core\Role\Role;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Security\Core\Security;

class TimelineCenterController extends AbstractController
{
    
    protected TimelineBuilder $timelineBuilder;
    
    protected PaginatorFactory $paginatorFactory;

    private Security $security;
    
    public function __construct(
        TimelineBuilder $timelineBuilder,
        PaginatorFactory $paginatorFactory,
        Security $security
    ) {
        $this->timelineBuilder = $timelineBuilder;
        $this->paginatorFactory = $paginatorFactory;
        $this->security = $security;
    }

    /**
     * @Route("/{_locale}/center/timeline",
     *    name="chill_center_timeline",
     *    methods={"GET"}
     *  )
     */ 
    public function centerAction(Request $request)
    {
        // collect reachable center for each group
        $user = $this->security->getUser();
        $centers = [];
        foreach ($user->getGroupCenters() as $group) {
            $centers[] = $group->getCenter();
        }

        if (0 === count($centers)) {
            throw $this->createNotFoundException();
        }
        
        $nbItems = $this->timelineBuilder->countItems('center', 
            [ 'centers' => $centers ]
            );
        
        $paginator = $this->paginatorFactory->create($nbItems);
        
        return $this->render('@ChillMain/Timeline/index.html.twig', array
            (
                'timeline' => $this->timelineBuilder->getTimelineHTML(
                    'center', 
                    [ 'centers' => $centers ],
                    $paginator->getCurrentPage()->getFirstItemNumber(),
                    $paginator->getItemsPerPage()
                    ),
                'nb_items' => $nbItems,
                'paginator' => $paginator
            )
        );
    }
}
