<?php

namespace Chill\MainBundle\Doctrine\Model;

use \JsonSerializable;

class Point implements JsonSerializable {
    private ?float $lat;
    private ?float $lon;
    public static string $SRID = '4326';

    private function __construct(?float $lon, ?float $lat)
    {
        $this->lat = $lat;
        $this->lon = $lon;
    }

    public function toGeoJson(): string
    {
        $array = $this->toArrayGeoJson();

        return \json_encode($array);
    }

    public function jsonSerialize(): array
    {
        return $this->toArrayGeoJson();
    }

    public function toArrayGeoJson(): array
    {
        return [
            'type' => 'Point',
            'coordinates' => [$this->lon, $this->lat],
        ];
    }

    public function toWKT(): string
    {
        return sprintf("SRID=%s;POINT(%s %s)", self::$SRID, $this->lon, $this->lat);
    }

    public static function fromGeoJson(string $geojson): self
    {
        $a = json_decode($geojson);

        if (null === $a) {
            throw PointException::badJsonString($geojson);
        }

        if (null === $a->type || null === $a->coordinates) {
            throw PointException::badJsonString($geojson);
        }

        if ($a->type !== 'Point'){
            throw PointException::badGeoType();
        }

        [$lon, $lat] = $a->coordinates;

        return Point::fromLonLat($lon, $lat);
    }

    public static function fromLonLat(float $lon, float $lat): self
    {
        if (($lon > -180 && $lon < 180) && ($lat > -90 && $lat < 90)) {
            return new Point($lon, $lat);
        }

        throw PointException::badCoordinates($lon, $lat);
    }

    public static function fromArrayGeoJson(array $array): self
    {
        if ($array['type'] === 'Point' && isset($array['coordinates'])) {
            return self::fromLonLat($array['coordinates'][0], $array['coordinates'][1]);
        }

        throw new \Exception('Unable to build a point from input data.');
    }

    public function getLat(): float
    {
        return $this->lat;
    }

    public function getLon(): float
    {
        return $this->lon;
    }
}


