<?php

declare(strict_types=1);

namespace Chill\MainBundle\Doctrine\Type;

use Chill\MainBundle\Doctrine\Model\Point;
use Doctrine\DBAL\Types\Type;
use Doctrine\DBAL\Platforms\AbstractPlatform;
use Chill\MainBundle\Doctrine\Model\PointException;

/**
 * A Type for Doctrine to implement the Geography Point type
 * implemented by Postgis on postgis+postgresql databases
 */
class PointType extends Type
{
    public const POINT = 'point';

    /**
     * @return string
     */
    public function getSQLDeclaration(array $column, AbstractPlatform $platform)
    {
        return 'geometry(POINT,'.Point::$SRID.')';
    }

    /**
     * @return ?Point
     */
    public function convertToPHPValue($value, AbstractPlatform $platform)
    {
        if ($value === NULL){
            return NULL;
        }

        return Point::fromGeoJson($value);
    }

    public function getName()
    {
        return self::POINT;
    }

    public function convertToDatabaseValue($value, AbstractPlatform $platform)
    {
        if ($value === NULL){
            return NULL;
        }

        return $value->toWKT();
    }

    public function canRequireSQLConversion()
    {
        return true;
    }

    public function convertToPHPValueSQL($sqlExpr, $platform)
    {
        return 'ST_AsGeoJSON('.$sqlExpr.') ';
    }

    public function convertToDatabaseValueSQL($sqlExpr, AbstractPlatform $platform)
    {
        return $sqlExpr;
    }
}

