<?php

namespace Chill\MainBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Chill\MainBundle\Doctrine\Model\Point;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 * @ORM\Entity
 * @ORM\Table(name="chill_main_address_reference")
 * @ORM\HasLifecycleCallbacks()
 */
class AddressReference
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     * @groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=255)
     * @groups({"read"})
     */
    private $refId;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @groups({"read"})
     */
    private $street;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @groups({"read"})
     */
    private $streetNumber;

    /**
     * @var PostalCode
     *
     * @ORM\ManyToOne(targetEntity="Chill\MainBundle\Entity\PostalCode")
     * @groups({"read"})
     */
    private $postcode;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @groups({"read"})
     */
    private $municipalityCode;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @groups({"read"})
     */
    private $source;

    /**
     * A geospatial field storing the coordinates of the Address
     *
     * @var Point
     *
     * @ORM\Column(type="point")
     * @groups({"read"})
     */
    private $point;


    public function getId(): ?int
    {
        return $this->id;
    }

    public function getRefId(): ?string
    {
        return $this->refId;
    }

    public function setRefId(string $refId): self
    {
        $this->refId = $refId;

        return $this;
    }

    public function getStreet(): ?string
    {
        return $this->street;
    }

    public function setStreet(?string $street): self
    {
        $this->street = $street;

        return $this;
    }

    public function getStreetNumber(): ?string
    {
        return $this->streetNumber;
    }

    public function setStreetNumber(?string $streetNumber): self
    {
        $this->streetNumber = $streetNumber;

        return $this;
    }

    /**
     * Set postcode
     *
     * @param PostalCode $postcode
     *
     * @return Address
     */
    public function setPostcode(PostalCode $postcode = null)
    {
        $this->postcode = $postcode;

        return $this;
    }

    /**
     * Get postcode
     *
     * @return PostalCode
     */
    public function getPostcode()
    {
        return $this->postcode;
    }

    public function getMunicipalityCode(): ?string
    {
        return $this->municipalityCode;
    }

    public function setMunicipalityCode(?string $municipalityCode): self
    {
        $this->municipalityCode = $municipalityCode;

        return $this;
    }

    public function getSource(): ?string
    {
        return $this->source;
    }

    public function setSource(?string $source): self
    {
        $this->source = $source;

        return $this;
    }

    public function getPoint(): ?Point
    {
        return $this->point;
    }

    public function setPoint(?Point $point): self
    {
        $this->point = $point;

        return $this;
    }
}
