<?php

/*
 * Chill is a suite of a modules, Chill is a software for social workers
 * Copyright (C) 2014, Champs Libres Cooperative SCRLFS, <http://www.champs-libres.coop>
 * 
 * This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\MainBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Chill\MainBundle\Entity\Center;
use Chill\MainBundle\Entity\PermissionsGroup;
use Doctrine\Common\Collections\Collection;
use Doctrine\Common\Collections\ArrayCollection;

/**
 * @ORM\Entity
 * @ORM\Table(name="group_centers")
 * @ORM\Cache(usage="NONSTRICT_READ_WRITE", region="acl_cache_region")
 *
 * @author Julien Fastré <julien.fastre@champs-libres.coop>
 */
class GroupCenter
{
    /**
     * @var int
     *
     * @ORM\Id
     * @ORM\Column(name="id", type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;
    
    /**
     * @var Center
     *
     * @ORM\ManyToOne(
     *     targetEntity="Chill\MainBundle\Entity\Center",
     *     inversedBy="groupCenters"
     * )
     * @ORM\Cache(usage="NONSTRICT_READ_WRITE")
     */
    private $center;
    
    /**
     * @var Collection
     *
     * @ORM\ManyToMany(
     *     targetEntity="Chill\MainBundle\Entity\User",
     *     mappedBy="groupCenters"
     * )
     */
    private $users;
    
    /**
     * @var PermissionsGroup
     *
     * @ORM\ManyToOne(
     *     targetEntity="Chill\MainBundle\Entity\PermissionsGroup",
     *     inversedBy="groupCenters")
     * @ORM\Cache(usage="NONSTRICT_READ_WRITE")
     */
    private $permissionsGroup;
    
    
    /**
     * GroupCenter constructor.
     */
    public function __construct()
    {
        $this->permissionsGroup = new ArrayCollection();
        $this->users = new ArrayCollection();
    }
    
    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return Center
     */
    public function getCenter()
    {
        return $this->center;
    }

    /**
     * @param Center $center
     * @return \Chill\MainBundle\Entity\GroupCenter
     */
    public function setCenter(Center $center)
    {
        $this->center = $center;
        return $this;
    }
    
    /**
     * @return ArrayCollection|Collection
     */
    public function getUsers()
    {
        return $this->users;
    }
    
    /**
     * @return PermissionGroup
     */
    public function getPermissionsGroup()
    {
        return $this->permissionsGroup;
    }

    /**
     * @param \Chill\MainBundle\Entity\PermissionsGroup $permissionGroup
     * @return \Chill\MainBundle\Entity\GroupCenter
     */
    public function setPermissionsGroup(PermissionsGroup $permissionsGroup)
    {
        $this->permissionsGroup = $permissionsGroup;
        return $this;
    }





}
