<?php

namespace Chill\MainBundle\Entity;

use Chill\MainBundle\Doctrine\Model\Point;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation\Groups;

/**
 * PostalCode
 *
 * @ORM\Entity
 * @ORM\Table(
 *     name="chill_main_postal_code",
 *     indexes={@ORM\Index(
 *          name="search_name_code",
 *          columns={"code", "label"}
 * )})
 * @ORM\HasLifecycleCallbacks()
 *
 */
class PostalCode
{
    /**
     * @var integer
     *
     * @ORM\Id
     * @ORM\Column(name="id", type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     * @groups({"write", "read"})
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=255, name="label")
     * @groups({"write", "read"})
     */
    private $name;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=100)
     * @groups({"write", "read"})
     */
    private $code;

    /**
     * @var Country
     *
     * @ORM\ManyToOne(targetEntity="Chill\MainBundle\Entity\Country")
     * @groups({"write", "read"})
     */
    private $country;

    /**
     * @var integer
     *
     * @ORM\Column(name="origin", type="integer", nullable=true)
     * @groups({"write", "read"})
     */
    private $origin = 0;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=255, nullable=true)
     * @groups({"read"})
     */
    private $refPostalCodeId;

    /**
     * @var string
     *
     * @ORM\Column(type="string", length=255, nullable=true)
     * @groups({"read"})
     */
    private $postalCodeSource;

    /**
     * @var Point
     *
     * @ORM\Column(type="point", nullable=true)
     * @groups({"read"})
     */
    private $center;

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }


    /**
     * Set origin
     *
     * @param int $origin
     *
     * @return PostalCode
     */
    public function setOrigin($origin)
    {
        $this->origin = $origin;

        return $this;
    }

    /**
     * Get origin
     *
     * @return int
     */
    public function getOrigin()
    {
        return $this->origin;
    }


    /**
     * Set name
     *
     * @param string $name
     *
     * @return PostalCode
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set code
     *
     * @param string $code
     *
     * @return PostalCode
     */
    public function setCode($code)
    {
        $this->code = $code;

        return $this;
    }

    /**
     * Get code
     *
     * @return string
     */
    public function getCode()
    {
        return $this->code;
    }

    /**
     * Set country
     *
     * @param Country $country
     *
     * @return PostalCode
     */
    public function setCountry(Country $country = null)
    {
        $this->country = $country;

        return $this;
    }

    /**
     * Get country
     *
     * @return Country
     */
    public function getCountry()
    {
        return $this->country;
    }

    public function getRefPostalCodeId(): ?string
    {
        return $this->refPostalCodeId;
    }

    public function setRefPostalCodeId(?string $refPostalCodeId): self
    {
        $this->refPostalCodeId = $refPostalCodeId;

        return $this;
    }

    public function getPostalCodeSource(): ?string
    {
        return $this->postalCodeSource;
    }

    public function setPostalCodeSource(?string $postalCodeSource): self
    {
        $this->postalCodeSource = $postalCodeSource;

        return $this;
    }

    public function getCenter(): ?Point
    {
        return $this->center;
    }

    public function setCenter(?Point $center): self
    {
        $this->center = $center;

        return $this;
    }
}

