<?php

namespace Chill\MainBundle\Form\Type\DataTransformer;

use Chill\MainBundle\Repository\AddressRepository;
use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\Form\Exception\TransformationFailedException;

final class AddressToIdDataTransformer implements DataTransformerInterface
{
    private AddressRepository $addressRepository;

    public function __construct(AddressRepository $addressRepository)
    {
        $this->addressRepository = $addressRepository;
    }

    public function reverseTransform($value)
    {
        if (NULL === $value || '' === $value) {
            return null;
        }

        $address = $this->addressRepository->find($value);

        if (NULL === $address) {
            $failure = new TransformationFailedException(sprintf("Address with id %s does not exists", $value));
            $failure
                ->setInvalidMessage("The given {{ value }} is not a valid address id", [ '{{ value }}' => $value]);

            throw $failure;
        }

        return $address;
    }

    public function transform($value)
    {
        if (NULL === $value) {
            return '';
        }

        return $value->getId();
    }
}
