<?php

/*
 * Copyright (C) 2015 Julien Fastré <julien.fastre@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\MainBundle\Form\Type\DataTransformer;

use Chill\MainBundle\Entity\Center;
use Chill\MainBundle\Repository\CenterRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\Form\Exception\TransformationFailedException;
use Symfony\Component\Form\Exception\UnexpectedTypeException;

class CenterTransformer implements DataTransformerInterface
{
    private CenterRepository $centerRepository;
    private bool $multiple = false;

    public function __construct(
        CenterRepository $centerRepository,
        bool $multiple = false
    ) {
        $this->centerRepository = $centerRepository;
        $this->multiple = $multiple;
    }

    public function reverseTransform($id)
    {
        if ($id === NULL) {
            if ($this->multiple) {
                return new ArrayCollection();
            } else {
                return NULL;
            }
        }

        $ids = [];

        if ($this->multiple) {
            $ids = explode(',', $id);
        } else {
            $ids[] = (int) $id;
        }

        $centers = $this
            ->centerRepository
            ->findBy(['id' => $ids ]);

        if ([] === $centers || count($ids) > count($centers)) {
            throw new TransformationFailedException(sprintf(
                    'No center found for one of those ids: %s', implode(',', $ids)));
        }

        if ($this->multiple) {
            return new ArrayCollection($centers);
        }

        return $centers[0];
    }

    public function transform($center)
    {
        if ($center === NULL) {
            return '';
        }

        if ($this->multiple) {
            if (!is_iterable($center)) {
                throw new UnexpectedTypeException($center, \Traversable::class);
            }
            $ids = [];
            foreach ($center as $c) {
                $ids[] = $c->getId();
            }

            return implode(',', $ids);
        } else {
            if (!$center instanceof Center) {
                throw new UnexpectedTypeException($center, Center::class);
            }
            return (string) $center->getId();
        }
    }
}
