<?php

/*
 * Chill is a software for social workers
 * Copyright (C) 2014 Champs-Libres Coopérative <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\MainBundle\Form\Type\DataTransformer;

use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\Form\Exception\TransformationFailedException;
use Doctrine\Persistence\ObjectManager;

class ObjectToIdTransformer implements DataTransformerInterface
{
    private EntityManagerInterface $em;

    private ?string $class;

    public function __construct(EntityManagerInterface $em, ?string $class = null)
    {
        $this->em = $em;
        $this->class = $class;
    }

    /**
     * Transforms an object to a string (id)
     *
     * @param  Object|null $Object
     * @return string
     */
    public function transform($object)
    {
        if (!$object) {
            return "";
        }

        return $object->getId();
    }

    /**
     * Transforms a string (id) to an object
     *
     * @param  string $id
     * @return Object|null
     * @throws TransformationFailedException if object is not found.
     */
    public function reverseTransform($id)
    {
        if (!$id) {
            return null;
        }

        $object = $this->em
            ->getRepository($this->class)
            ->find($id)
        ;

        if (! $object) {
            throw new TransformationFailedException();
        }

        return $object;
    }
}