<?php

/*
 * Chill is a software for social workers
 *
 * Copyright (C) 2014-2015, Champs Libres Cooperative SCRLFS,
 * <http://www.champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\MainBundle\Form\Type\Export;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Form\FormBuilderInterface;
use Chill\MainBundle\Export\ExportManager;
use Chill\MainBundle\Form\Type\Export\FilterType;
use Chill\MainBundle\Form\Type\Export\AggregatorType;
use Symfony\Component\Form\Extension\Core\Type\FormType;
use Chill\MainBundle\Validator\Constraints\Export\ExportElementConstraint;
use Chill\MainBundle\Export\ExportElementWithValidationInterface;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

/**
 *
 *
 * @author Julien Fastré <julien.fastre@champs-libres.coop>
 */
class ExportType extends AbstractType
{
    /**
     *
     * @var ExportManager
     */
    protected $exportManager;

    const FILTER_KEY = 'filters';
    const AGGREGATOR_KEY = 'aggregators';
    const PICK_FORMATTER_KEY = 'pick_formatter';
    const EXPORT_KEY = 'export';

    public function __construct(ExportManager $exportManager)
    {
        $this->exportManager = $exportManager;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $export = $this->exportManager->getExport($options['export_alias']);

        $exportOptions = array(
            'compound' => true,
            'constraints' => array(

            )
        );

        // add a contraint if required by export
        $exportBuilder = $builder->create(self::EXPORT_KEY/*, FormType::class, $exportOptions*/);

        $export->buildForm($exportBuilder);

        $builder->add($exportBuilder, null, $exportOptions);

        if ($export instanceof \Chill\MainBundle\Export\ExportInterface) {
            //add filters
            $filters = $this->exportManager->getFiltersApplyingOn($export, $options['picked_centers']);
            $filterBuilder = $builder->create(self::FILTER_KEY, FormType::class, array('compound' => true));

            foreach($filters as $alias => $filter) {
                $filterBuilder->add($alias, FilterType::class, array(
                    'filter_alias' => $alias,
                    'export_manager' => $this->exportManager,
                    'label' => $filter->getTitle(),
                    'constraints' => array(
                        new ExportElementConstraint(['element' => $filter])
                    )
                ));
            }

            $builder->add($filterBuilder);

            //add aggregators
            $aggregators = $this->exportManager
                    ->getAggregatorsApplyingOn($export, $options['picked_centers']);
            $aggregatorBuilder = $builder->create(self::AGGREGATOR_KEY, FormType::class,
                    array('compound' => true));

            foreach($aggregators as $alias => $aggregator) {
                $aggregatorBuilder->add($alias, AggregatorType::class, array(
                    'aggregator_alias' => $alias,
                    'export_manager' => $this->exportManager,
                    'label' => $aggregator->getTitle(),
                    'constraints' => array(
                        new ExportElementConstraint(['element' => $aggregator])
                    )
                ));
            }

            $builder->add($aggregatorBuilder);
        }

        // add export form
        $exportBuilder = $builder->create(self::EXPORT_KEY, FormType::class, array('compound' => true));
        $this->exportManager->getExport($options['export_alias'])
              ->buildForm($exportBuilder);
        $builder->add($exportBuilder);

        if ($export instanceof \Chill\MainBundle\Export\ExportInterface) {
            $builder->add(self::PICK_FORMATTER_KEY, PickFormatterType::class, array(
                'export_alias' => $options['export_alias']
            ));
        }

    }


    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setRequired(array('export_alias', 'picked_centers'))
                ->setAllowedTypes('export_alias', array('string'))
                ->setDefault('compound', true)
                ->setDefault('constraints', array(
                    //new \Chill\MainBundle\Validator\Constraints\Export\ExportElementConstraint()
                ))
                ;

    }

    public function getParent()
    {
        return FormType::class;
    }
}
