<?php

/*
 * Chill is a software for social workers
 * Copyright (C) 2014, Champs Libres Cooperative SCRLFS, <http://www.champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\MainBundle\Form\Type;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\Form\FormBuilderInterface;
use Chill\MainBundle\Form\Type\DataTransformer\MultipleObjectsToIdTransformer;
use Doctrine\Persistence\ObjectManager;
use Chill\MainBundle\Form\Type\Select2ChoiceType;
use Chill\MainBundle\Templating\TranslatableStringHelper;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;

/**
 * Extends choice to allow adding select2 library on widget for languages (multiple)
 */
class Select2LanguageType extends AbstractType
{
    private RequestStack $requestStack;

    private ObjectManager $em;

    protected TranslatableStringHelper $translatableStringHelper;

    protected ParameterBagInterface $parameterBag;

    public function __construct(
        RequestStack $requestStack,
        ObjectManager $em,
        TranslatableStringHelper $translatableStringHelper,
        ParameterBagInterface $parameterBag
        )
    {
        $this->requestStack = $requestStack;
        $this->em = $em;
        $this->translatableStringHelper = $translatableStringHelper;
        $this->parameterBag = $parameterBag;
    }

    public function getBlockPrefix()
    {
        return 'select2_chill_language';
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $transformer = new MultipleObjectsToIdTransformer($this->em,'Chill\MainBundle\Entity\Language');
        $builder->addModelTransformer($transformer);
    }

    public function getParent()
    {
        return Select2ChoiceType::class;
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $languages = $this->em->getRepository('Chill\MainBundle\Entity\Language')->findAll();
        $preferredLanguages = $this->parameterBag->get('chill_main.available_languages');
        $choices = [];
        $preferredChoices = [];

        foreach ($languages as $l) {
            $choices[$l->getId()] = $this->translatableStringHelper->localize($l->getName());
        }
        foreach ($preferredLanguages as $l) {
            $preferredChoices[$l] = $choices[$l];
        }

        asort($choices, SORT_STRING | SORT_FLAG_CASE);

        $resolver->setDefaults(array(
           'class' => 'Chill\MainBundle\Entity\Language',
           'choices' => array_combine(array_values($choices), array_keys($choices)),
           'preferred_choices' => array_combine(array_values($preferredChoices), array_keys($preferredChoices))
        ));
    }
}
