<?php

/*
 * Copyright (C) 2021 Champs-Libres <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\MainBundle\Repository;

use Chill\MainBundle\Entity\Notification;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\EntityRepository;
use Doctrine\Persistence\ObjectRepository;
use Chill\MainBundle\Entity\User;
use Doctrine\ORM\Query;

final class NotificationRepository implements ObjectRepository
{
    private EntityRepository $repository;

    public function __construct(EntityManagerInterface $entityManager)
    {
        $this->repository = $entityManager->getRepository(Notification::class);
    }

    public function find($id, $lockMode = null, $lockVersion = null): ?Notification
    {
        return $this->repository->find($id, $lockMode, $lockVersion);
    }

    public function findOneBy(array $criteria, array $orderBy = null): ?Notification
    {
        return $this->repository->findOneBy($criteria, $orderBy);
    }

    /**
     * @return Notification[]
     */
    public function findAll(): array
    {
        return $this->repository->findAll();
    }

    /**
     * @return Notification[]
     */
    public function findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null): array
    {
        return $this->repository->findBy($criteria, $orderBy, $limit, $offset);
    }

    private function queryAllForAttendee(User $addressee, bool $countQuery=False): Query
    {
        $qb = $this->repository->createQueryBuilder('n');

        $select = 'n';
        if($countQuery) {
            $select = 'count(n)';
        }

        $qb
            ->select($select)
            ->join('n.addressees', 'a')
            ->where('a = :addressee')
            ->setParameter('addressee', $addressee);

        return $qb->getQuery();
    }

    /**
     * @return int
     */
    public function  countAllForAttendee(User $addressee): int // TODO passer à attendees avec S
    {
        $query = $this->queryAllForAttendee($addressee, $countQuery=True);

        return $query->getSingleScalarResult();
    }


    /**
     * @return Notification[]
     */
    public function  findAllForAttendee(User $addressee, $limit = null, $offset = null): array // TODO passer à attendees avec S
    {
        $query = $this->queryAllForAttendee($addressee);

        if($limit) {
            $query = $query->setMaxResults($limit);
        }

        if($offset) {
            $query = $query->setFirstResult($offset);
        }

        return $query->getResult();
    }

    public function getClassName() {
        return Notification::class;
    }
}
