<?php

namespace Chill\MainBundle\Repository;

use Chill\MainBundle\Entity\Center;
use Chill\MainBundle\Entity\Scope;
use Chill\MainBundle\Entity\User;
use Chill\MainBundle\Security\Authorization\AuthorizationHelper;
use Chill\MainBundle\Security\ParentRoleHelper;
use Doctrine\ORM\EntityManagerInterface;

class UserACLAwareRepository implements UserACLAwareRepositoryInterface
{
    private ParentRoleHelper $parentRoleHelper;
    private EntityManagerInterface $em;

    public function __construct(ParentRoleHelper $parentRoleHelper, EntityManagerInterface $em)
    {
        $this->parentRoleHelper = $parentRoleHelper;
        $this->em = $em;
    }

    public function findUsersByReachedACL(string $role, $center, $scope = null, bool $onlyEnabled = true): array
    {
        $parents = $this->parentRoleHelper->getParentRoles($role);
        $parents[] = $role;

        $qb = $this->em->createQueryBuilder();

        $qb
            ->select('u')
            ->from(User::class, 'u')
            ->join('u.groupCenters', 'gc')
            ->join('gc.permissionsGroup', 'pg')
            ->join('pg.roleScopes', 'rs')
            ->where($qb->expr()->in('rs.role', $parents))
        ;

        if ($onlyEnabled) {
            $qb->andWhere($qb->expr()->eq('u.enabled', "'TRUE'"));
        }

        if (NULL !== $center) {
            $centers = $center instanceof Center ? [$center] : $center;
            $qb
                ->andWhere($qb->expr()->in('gc.center', ':centers'))
                ->setParameter('centers', $centers)
                ;
        }

        if (NULL !== $scope) {
            $scopes = $scope instanceof Scope ? [$scope] : $scope;
            $qb
                ->andWhere($qb->expr()->in('rs.scope', ':scopes'))
                ->setParameter('scopes', $scopes)
            ;
        }

        return $qb->getQuery()->getResult();
    }

}
