<?php

namespace Chill\MainBundle\Search\Entity;

use Chill\MainBundle\Repository\UserRepository;
use Chill\MainBundle\Search\SearchApiInterface;
use Chill\MainBundle\Search\SearchApiQuery;

class SearchUserApiProvider implements SearchApiInterface
{
    private UserRepository $userRepository;

    /**
     * @param UserRepository $userRepository
     */
    public function __construct(UserRepository $userRepository)
    {
        $this->userRepository = $userRepository;
    }

    public function provideQuery(string $pattern, array $parameters): SearchApiQuery
    {
        $query = new SearchApiQuery();
        $query
            ->setSelectKey("user")
            ->setSelectJsonbMetadata("jsonb_build_object('id', u.id)")
            ->setSelectPertinence("GREATEST(SIMILARITY(LOWER(UNACCENT(?)), u.usernamecanonical),
                SIMILARITY(LOWER(UNACCENT(?)), u.emailcanonical))", [ $pattern, $pattern ])
            ->setFromClause("users AS u")
            ->setWhereClauses("SIMILARITY(LOWER(UNACCENT(?)), u.usernamecanonical) > 0.15
                OR
                SIMILARITY(LOWER(UNACCENT(?)), u.emailcanonical) > 0.15
            ", [ $pattern, $pattern ]);

        return $query;
    }

    public function supportsTypes(string $pattern, array $types, array $parameters): bool
    {
        return \in_array('user', $types);
    }

    public function prepare(array $metadatas): void
    {
        $ids = \array_map(fn($m) => $m['id'], $metadatas);

        $this->userRepository->findBy([ 'id' => $ids ]);
    }

    public function supportsResult(string $key, array $metadatas): bool
    {
        return $key === 'user';
    }

    public function getResult(string $key, array $metadata, float $pertinence)
    {
        return $this->userRepository->find($metadata['id']);
    }
}
