<?php

namespace Chill\MainBundle\Security\Authorization;

use Chill\MainBundle\Entity\User;
use Chill\MainBundle\Security\Resolver\CenterResolverDispatcher;

final class DefaultVoterHelper implements VoterHelperInterface
{
    protected AuthorizationHelper $authorizationHelper;

    protected CenterResolverDispatcher $centerResolverDispatcher;

    protected array $configuration = [];

    /**
     * @param AuthorizationHelper $authorizationHelper
     * @param CenterResolverDispatcher $centerResolverDispatcher
     * @param array $configuration
     */
    public function __construct(
        AuthorizationHelper $authorizationHelper,
        CenterResolverDispatcher $centerResolverDispatcher,
        array $configuration
    ) {
        $this->authorizationHelper = $authorizationHelper;
        $this->centerResolverDispatcher = $centerResolverDispatcher;
        $this->configuration = $configuration;
    }

    public function supports($attribute, $subject): bool
    {
        foreach ($this->configuration as list($attributes, $subj)) {
            if ($subj === null) {
                if ($subject === null && \in_array($attribute, $attributes)) {
                    return true;
                }
            } elseif ($subject instanceof $subj) {
                return \in_array($attribute, $attributes);
            }
        }

        return false;
    }

    public function voteOnAttribute($attribute, $subject, $token): bool
    {
        if (!$token->getUser() instanceof User) {
            return false;
        }

        if (NULL === $subject) {
            return 0 < count($this->authorizationHelper->getReachableCenters($token->getUser(), $attribute, null));
        }

        return $this->authorizationHelper->userHasAccess(
            $token->getUser(),
            $subject,
            $attribute
        );
    }
}
