<?php

namespace Chill\MainBundle\Security\Resolver;

use Chill\MainBundle\Entity\Scope;

final class ScopeResolverDispatcher
{
    /**
     * @var iterable|ScopeResolverInterface[]
     */
    private iterable $resolvers;

    public function __construct(iterable $resolvers)
    {
        $this->resolvers = $resolvers;
    }

    /**
     * @param $entity
     * @return Scope|Scope[]|iterable
     */
    public function resolveScope($entity, ?array $options = [])
    {
        foreach ($this->resolvers as $resolver) {
            if ($resolver->supports($entity, $options)) {
                return $resolver->resolveScope($entity, $options);
            }
        }

        return null;
    }

    public function isConcerned($entity, ?array $options = []): bool
    {
        foreach ($this->resolvers as $resolver) {
            if ($resolver->supports($entity, $options)) {
                return $resolver->isConcerned($entity, $options);
            }
        }

        return false;
    }
}
