<?php

namespace Chill\MainBundle\Security\Resolver;

use Chill\MainBundle\Entity\Scope;

/**
 * Interface to implement to define a ScopeResolver.
 */
interface ScopeResolverInterface
{
    /**
     * Return true if this resolve is able to decide "something" on this entity.
     */
    public function supports($entity, ?array $options = []): bool;

    /**
     * Will return the scope for the entity
     *
     * @return Scope|array|Scope[]
     */
    public function resolveScope($entity, ?array $options = []);


    /**
     * Return true if the entity is concerned by scope, false otherwise.
     */
    public function isConcerned($entity, ?array $options = []): bool;

    /**
     * get the default priority for this resolver. Resolver with an higher priority will be
     * queried first.
     */
    public static function getDefaultPriority(): int;
}
