<?php

declare(strict_types=1);

namespace Chill\MainBundle\Security\UserProvider;

use Doctrine\ORM\NoResultException;
use Symfony\Component\Security\Core\User\UserProviderInterface;
use Symfony\Component\Security\Core\User\UserInterface;
use Doctrine\ORM\EntityManagerInterface;
use Chill\MainBundle\Entity\User;
use Symfony\Component\Security\Core\Exception\UsernameNotFoundException;
use Symfony\Component\Security\Core\Exception\UnsupportedUserException;

class UserProvider implements UserProviderInterface
{
    protected EntityManagerInterface $em;

    public function __construct(EntityManagerInterface $em)
    {
        $this->em = $em;
    }

    public function loadUserByUsername($username): UserInterface
    {
        try {
            $user = $this->em->createQuery(sprintf(
                    "SELECT u FROM %s u "
                    . "WHERE u.usernameCanonical = UNACCENT(LOWER(:pattern)) "
                    . "OR "
                    . "u.emailCanonical = UNACCENT(LOWER(:pattern))",
                    User::class))
                ->setParameter('pattern', $username)
                ->getSingleResult();
        } catch (NoResultException $e) {
            throw new UsernameNotFoundException(
                sprintf('Bad credentials.'),
                0,
                $e
            );
        }

        return $user;
    }

    public function refreshUser(UserInterface $user): UserInterface
    {
        if (!$user instanceof User) {
            throw new UnsupportedUserException("Unsupported user class: cannot reload this user");
        }

        $reloadedUser = $this->em->getRepository(User::class)->find($user->getId());

        if (NULL === $reloadedUser) {
            throw new UsernameNotFoundException(sprintf('User with ID "%s" could not be reloaded.', $user->getId()));
        }

        return $reloadedUser;
    }

    public function supportsClass($class): bool
    {
        return $class === User::class;
    }
}
