<?php

namespace Chill\MainBundle\Serializer\Normalizer;

use Chill\MainBundle\Serializer\Model\Collection;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerAwareInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerAwareTrait;

class CollectionNormalizer implements NormalizerInterface, NormalizerAwareInterface
{
    use NormalizerAwareTrait;

    /**
     * @param Collection $collection
     */
    public function normalize($collection, string $format = null, array $context = [])
    {
        $paginator = $collection->getPaginator();

        return [
            'count' => $paginator->getTotalItems(),
            'pagination' => [
                'first' => $paginator->getCurrentPageFirstItemNumber(),
                'items_per_page' => $paginator->getItemsPerPage(),
                'next' => $paginator->hasNextPage() ? $paginator->getNextPage()->generateUrl() : null,
                'previous' => $paginator->hasPreviousPage() ? $paginator->getPreviousPage()->generateUrl() : null,
                'more' => $paginator->hasNextPage(),
            ],
            'results' => $this->normalizer->normalize($collection->getItems(), $format, $context),
        ];
    }

    public function supportsNormalization($data, string $format = null): bool
    {
        return $data instanceof Collection;
    }
}
