<?php
/*
 *
 * Copyright (C) 2014-2021, Champs Libres Cooperative SCRLFS, <http://www.champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\MainBundle\Serializer\Normalizer;

use Chill\MainBundle\Entity\User;
use Chill\MainBundle\Templating\Entity\UserRender;
use Symfony\Component\Serializer\Normalizer\NormalizerAwareInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerAwareTrait;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;

class UserNormalizer implements NormalizerInterface, NormalizerAwareInterface
{
    use NormalizerAwareTrait;
    private UserRender $userRender;

    public function __construct(UserRender $userRender)
    {
        $this->userRender = $userRender;
    }

    public function normalize($user, string $format = null, array $context = array())
    {
        /** @var User $user */
        return [
            'type' => 'user',
            'id' => $user->getId(),
            'username' => $user->getUsername(),
            'text' => $this->userRender->renderString($user, []),
            'label' => $user->getLabel(),
            'user_job' => $this->normalizer->normalize($user->getUserJob(), $format, $context),
            'main_center' => $this->normalizer->normalize($user->getMainCenter(), $format, $context),
            'main_scope' => $this->normalizer->normalize($user->getMainScope(), $format, $context),
        ];
    }

    public function supportsNormalization($data, string $format = null): bool
    {
        return $format === 'json' && $data instanceof User;
    }
}
