<?php
/*
 * Chill is a software for social workers
 *
 * Copyright (C) 2014-2020, Champs Libres Cooperative SCRLFS,
 * <http://www.champs-libres.coop>, <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
namespace Chill\MainBundle\Templating\Entity;

use Chill\MainBundle\Entity\Embeddable\CommentEmbeddable;
use Chill\MainBundle\Repository\UserRepository;
use Chill\MainBundle\Templating\Entity\AbstractChillEntityRender;
use Symfony\Component\Templating\EngineInterface;

class CommentRender extends AbstractChillEntityRender
{
    /**
     * @var \Chill\MainBundle\Repository\UserRepository
     */
    private $userRepository;
    
    /**
     * 
     * @var EngineInterface
     */
    private $engine;

    public function __construct(
        UserRepository $userRepository,
        EngineInterface $engine
    ) {
        $this->userRepository = $userRepository;
        $this->engine = $engine;
    }

    /**
     * @param CommentEmbeddable $entity
     * @param array $options
     *
     * @return string
     */
    public function renderBox($entity, array $options): string
    {
        // default options
        $options = \array_merge([
                'user' => [],
                'disable_markdown' => false,
                'limit_lines' => null,
                'metadata' => true
            ], $options);
        
        if ($entity->getUserId()) {
            $user = $this->userRepository->find($entity->getUserId());
        }
        
        return $this->engine
            ->render(
                '@ChillMain/Entity/CommentEmbeddable.html.twig',
                [
                    'opening_box' => $this->getDefaultOpeningBox('comment-embeddable'),
                    'closing_box' => $this->getDefaultClosingBox(),
                    'user' => $user ?? NULL,
                    'comment' => $entity,
                    'options' => $options
                ]
            );
    }

    /**
     * @param CommentEmbeddable $entity
     * @param array $options
     *
     * @return string
     */
    public function renderString($entity, array $options): string
    {
        return $entity->getComment();
    }

    public function supports($entity, array $options): bool
    {
        return $entity instanceof CommentEmbeddable;
    }
}
