<?php

namespace Chill\MainBundle\Templating\Entity;

use Chill\MainBundle\Entity\User;
use Chill\MainBundle\Templating\TranslatableStringHelper;
use Symfony\Component\Templating\EngineInterface;

class UserRender implements ChillEntityRenderInterface
{
    private TranslatableStringHelper $translatableStringHelper;
    private EngineInterface $engine;

    const DEFAULT_OPTIONS = [
        'main_scope' => true,
        'user_job' => true
    ];

    public function __construct(TranslatableStringHelper $translatableStringHelper, EngineInterface $engine)
    {
        $this->translatableStringHelper = $translatableStringHelper;
        $this->engine = $engine;
    }

    /**
     * @inheritDoc
     */
    public function supports($entity, array $options): bool
    {
        return $entity instanceof User;
    }

    /**
     * @inheritDoc
     * @param User $entity
     */
    public function renderString($entity, array $options): string
    {
        $opts = \array_merge(self::DEFAULT_OPTIONS, $options);

        $str = $entity->getLabel();
        if (NULL !== $entity->getUserJob() && $opts['user_job']) {
            $str .= ' ('.$this->translatableStringHelper
                ->localize($entity->getUserJob()->getLabel()).')';
        }
        if (NULL !== $entity->getMainScope() && $opts['main_scope']) {
            $str .= ' ('.$this->translatableStringHelper
                    ->localize($entity->getMainScope()->getName()).')';
        }

        return $str;
    }

    /**
     * @inheritDoc
     */
    public function renderBox($entity, array $options): string
    {
        $opts = \array_merge(self::DEFAULT_OPTIONS, $options);

        return $this->engine->render('@ChillMain/Entity/user.html.twig', [
            'user' => $entity,
            'opts' => $opts
        ]);
    }
}
