<?php

namespace Chill\MainBundle\Templating\Listing;

use Chill\MainBundle\Form\Type\Listing\FilterOrderType;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class FilterOrderHelper
{
    private FormFactoryInterface $formFactory;
    private RequestStack $requestStack;
    private ?array $searchBoxFields = null;
    private array $checkboxes = [];
    private ?array $submitted = null;
    private ?string $formName = 'f';
    private string $formType = FilterOrderType::class;
    private array $formOptions = [];


    public function __construct(
        FormFactoryInterface $formFactory,
        RequestStack $requestStack
    ) {
        $this->formFactory = $formFactory;
        $this->requestStack = $requestStack;
    }

    public function setSearchBox($searchBoxFields = null): self
    {
        $this->searchBoxFields = $searchBoxFields;

        return $this;
    }

    public function addCheckbox(string $name, array $choices, ?array $default = [], ?array $trans = []): self
    {
        $missing = count($choices) - count($trans) - 1;
        $this->checkboxes[$name] = [
            'choices' => $choices, 'default' => $default,
            'trans' =>
                \array_merge(
                    $trans,
                    0 < $missing ?
                       array_fill(0, $missing, null) : []
                )
        ];

        return $this;
    }

    public function getCheckboxData(string $name): array
    {
        return $this->getFormData()['checkboxes'][$name];
    }

    public function getCheckboxes(): array
    {
        return $this->checkboxes;
    }

    public function hasSearchBox(): bool
    {
        return $this->searchBoxFields !== null;
    }

    private function getFormData(): array
    {
        if (NULL === $this->submitted) {
           $this->submitted = $this->buildForm()
               ->getData();
        }

        return $this->submitted;
    }

    private function getDefaultData(): array
    {
       $r = [];

       if ($this->hasSearchBox()) {
           $r['q'] = '';
       }
        foreach ($this->checkboxes as $name => $c) {
            $r['checkboxes'][$name] = $c['default'];
        }

        return $r;
    }

    public function getQueryString(): ?string
    {
        return $this->getFormData()['q'];
    }

    public function buildForm(): FormInterface
    {
        return $this->formFactory
            ->createNamed($this->formName, $this->formType, $this->getDefaultData(), \array_merge([
                'helper' => $this,
                'method' => 'GET',
                'csrf_protection' => false,
            ], $this->formOptions))
            ->handleRequest($this->requestStack->getCurrentRequest());
    }
}
