<?php

namespace Chill\MainBundle\Templating\Listing;

use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class FilterOrderHelperBuilder
{
    private ?array $searchBoxFields = null;
    private array $checkboxes = [];
    private FormFactoryInterface $formFactory;
    private RequestStack $requestStack;

    public function __construct(
        FormFactoryInterface $formFactory,
        RequestStack $requestStack
    ) {
        $this->formFactory = $formFactory;
        $this->requestStack = $requestStack;
    }

    public function addSearchBox(?array $fields = [], ?array $options = []): self
    {
        $this->searchBoxFields = $fields;

        return $this;
    }

    public function addCheckbox(string $name, array $choices, ?array $default = [], ?array $trans = []): self
    {
        $this->checkboxes[$name] = [ 'choices' => $choices, 'default' => $default, 'trans' => $trans];

        return $this;
    }

    public function build(): FilterOrderHelper
    {
        $helper = new FilterOrderHelper(
            $this->formFactory,
            $this->requestStack
        );

        $helper->setSearchBox($this->searchBoxFields);
        foreach ($this->checkboxes as $name => [
            'choices' => $choices,
            'default' => $default,
            'trans' => $trans
        ]) {
           $helper->addCheckbox($name, $choices, $default, $trans);
        }

        return $helper;
    }
}
