<?php

declare(strict_types=1);

namespace Chill\MainBundle\Templating;

use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Contracts\Translation\TranslatorInterface;

final class TranslatableStringHelper implements TranslatableStringHelperInterface
{
    private RequestStack $requestStack;

    private TranslatorInterface $translator;

    public function __construct(RequestStack $requestStack, TranslatorInterface $translator)
    {
        $this->requestStack = $requestStack;
        $this->translator = $translator;
    }

    public function localize(array $translatableStrings): ?string
    {
        if ([] === $translatableStrings) {
            return null;
        }

        $request = $this->requestStack->getCurrentRequest();

        if (null === $request) {
            return null;
        }

        $language = $request->getLocale();

        if (array_key_exists($language, $translatableStrings)) {
            return $translatableStrings[$language];
        }

        foreach ($this->translator->getFallbackLocales() as $locale) {
            if (array_key_exists($locale, $translatableStrings)) {
                return $translatableStrings[$locale];
            }
        }

        // no fallback translation... trying the  first available
        $langs = array_keys($translatableStrings);

        if ([] === $langs) {
            return '';
        }

        return $translatableStrings[$langs[0]];
    }

}
