<?php

namespace Chill\PersonBundle\Controller;

use Chill\MainBundle\Serializer\Model\Collection;
use Chill\MainBundle\Pagination\PaginatorFactory;
use Chill\PersonBundle\Repository\SocialWork\SocialIssueRepository;
use Chill\MainBundle\CRUD\Controller\ApiController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;

class SocialWorkSocialActionApiController extends ApiController
{
    private SocialIssueRepository $socialIssueRepository;

    private PaginatorFactory $paginator;

    /**
     * @param SocialIssueRepository $socialIssueRepository
     */
    public function __construct(SocialIssueRepository $socialIssueRepository, PaginatorFactory $paginator)
    {
        $this->socialIssueRepository = $socialIssueRepository;
        $this->paginator = $paginator;
    }
    
    public function listBySocialIssueApi($id, Request $request)
    {
        $socialIssue = $this->socialIssueRepository
            ->find($id);

        if (NULL === $socialIssue) {
            throw $this->createNotFoundException("socialIssue not found");
        }

        $socialActions = $socialIssue->getRecursiveSocialActions();
        $pagination = $this->paginator->create(count($socialActions));
        // max one page
        $pagination->setItemsPerPage(count($socialActions));

        $collection = new Collection($socialActions, $pagination);
        

        return $this->json($collection, JsonResponse::HTTP_OK, [], [ "groups" => [ "read" ]]);
    } 

}
