<?php
declare(strict_types=1);

namespace Chill\PersonBundle\DataFixtures\ORM;

use Chill\MainBundle\DataFixtures\ORM\LoadUsers;
use Chill\MainBundle\Entity\User;
use Chill\PersonBundle\DataFixtures\Helper\PersonRandomHelper;
use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\Common\DataFixtures\DependentFixtureInterface;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\Persistence\ObjectManager;
use Chill\PersonBundle\Entity\Relationships\Relationship;

class LoadRelationships extends Fixture implements DependentFixtureInterface
{
    use PersonRandomHelper;
    private EntityManagerInterface $em;

    public function __construct(EntityManagerInterface $em)
    {
        $this->em = $em;
    }

    public function getDependencies()
    {
        return [
            LoadPeople::class,
            LoadRelations::class
        ];
    }

    public function load(ObjectManager $manager)
    {
        for ($i = 0; $i < 15; $i++) {
            $user = $this->getRandomUser();
            $date = new \DateTimeImmutable();
            $relationship = (new Relationship())
                ->setFromPerson($this->getRandomPerson($this->em))
                ->setToPerson($this->getRandomPerson($this->em))
                ->setRelation($this->getReference(LoadRelations::RELATION_KEY.
                    \random_int(0, count(LoadRelations::RELATIONS) - 1)))
                ->setReverse((bool) random_int(0, 1))
                ->setCreatedBy($user)
                ->setUpdatedBy($user)
                ->setCreatedAt($date)
                ->setUpdatedAt($date)
            ;
            $manager->persist($relationship);
        }

        $manager->flush();
    }

    private function getRandomUser(): User
    {
        $userRef = array_rand(LoadUsers::$refs);
        return $this->getReference($userRef);
    }
}
