<?php

namespace Chill\PersonBundle\Entity\AccompanyingPeriod;

use Chill\DocGeneratorBundle\Entity\DocGeneratorTemplate;
use Chill\DocStoreBundle\Entity\StoredObject;
use Chill\MainBundle\Doctrine\Model\TrackCreationInterface;
use Chill\MainBundle\Doctrine\Model\TrackUpdateInterface;
use Chill\MainBundle\Entity\User;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation as Serializer;

/**
 * @ORM\Entity
 * @ORM\Table("chill_person_accompanying_period_work_evaluation_document")
 * @Serializer\DiscriminatorMap(typeProperty="type", mapping={
 *     "accompanying_period_work_evaluation_document"=AccompanyingPeriodWorkEvaluationDocument::class
 *     })
 */
class AccompanyingPeriodWorkEvaluationDocument implements \Chill\MainBundle\Doctrine\Model\TrackCreationInterface, \Chill\MainBundle\Doctrine\Model\TrackUpdateInterface
{
    /**
     * @var int|null
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     *
     * @internal the default name exceeds 64 characters, we must set manually:
     * @ORM\SequenceGenerator(sequenceName="chill_person_social_work_eval_doc_id_seq", allocationSize=1, initialValue=1000)
     * @Serializer\Groups({"read"})
     */
    private ?int $id;

    /**
     * @var AccompanyingPeriodWorkEvaluation|null
     * @ORM\ManyToOne(
     *     targetEntity=AccompanyingPeriodWorkEvaluation::class,
     *     inversedBy="documents"
     * )
     */
    private ?AccompanyingPeriodWorkEvaluation $accompanyingPeriodWorkEvaluation = null;

    /**
     * @ORM\ManyToOne(
     *     targetEntity=User::class
     * )
     * @Serializer\Groups({"read"})
     */
    private ?User $createdBy = null;

    /**
     * @ORM\Column(type="date_immutable", nullable=true, options={"default": null})
     * @Serializer\Groups({"read"})
     */
    private ?\DateTimeImmutable $createdAt = null;

    /**
     * @ORM\ManyToOne(
     *     targetEntity=User::class
     * )
     * @Serializer\Groups({"read"})
     */
    private ?User $updatedBy = null;

    /**
     * @ORM\Column(type="date_immutable", nullable=true, options={"default": null})
     * @Serializer\Groups({"read"})
     */
    private ?\DateTimeImmutable $updatedAt = null;

    /**
    * @ORM\ManyToOne(
    *     targetEntity=StoredObject::class,
    *     cascade={"remove"},
    * )
     * @Serializer\Groups({"read"})
    */
    private ?StoredObject $storedObject = null;

    /**
     * @ORM\ManyToOne(
     *     targetEntity=DocGeneratorTemplate::class
     * )
     * @Serializer\Groups({"read"})
     */
    private ?DocGeneratorTemplate $template = null;

    /**
     * @return AccompanyingPeriodWorkEvaluation|null
     */
    public function getAccompanyingPeriodWorkEvaluation(): ?AccompanyingPeriodWorkEvaluation
    {
        return $this->accompanyingPeriodWorkEvaluation;
    }

    /**
     * @param AccompanyingPeriodWorkEvaluation|null $accompanyingPeriodWorkEvaluation
     * @return AccompanyingPeriodWorkEvaluationDocument
     */
    public function setAccompanyingPeriodWorkEvaluation(?AccompanyingPeriodWorkEvaluation $accompanyingPeriodWorkEvaluation): AccompanyingPeriodWorkEvaluationDocument
    {
        // if an evaluation is already associated, we cannot change the association (removing the association,
        // by setting a null value, is allowed.
        if ($this->accompanyingPeriodWorkEvaluation instanceof AccompanyingPeriodWorkEvaluation
            && $accompanyingPeriodWorkEvaluation instanceof AccompanyingPeriodWorkEvaluation) {
            if ($this->accompanyingPeriodWorkEvaluation !== $accompanyingPeriodWorkEvaluation) {
                throw new \RuntimeException("It is not allowed to change the evaluation for a document");
            }
        }
        $this->accompanyingPeriodWorkEvaluation = $accompanyingPeriodWorkEvaluation;
        return $this;
    }

    /**
     * @return StoredObject|null
     */
    public function getStoredObject(): ?StoredObject
    {
        return $this->storedObject;
    }

    /**
     * @param StoredObject|null $storedObject
     * @return AccompanyingPeriodWorkEvaluationDocument
     */
    public function setStoredObject(?StoredObject $storedObject): AccompanyingPeriodWorkEvaluationDocument
    {
        $this->storedObject = $storedObject;
        return $this;
    }

    public function setCreatedBy(User $user): \Chill\MainBundle\Doctrine\Model\TrackCreationInterface
    {
        $this->createdBy = $user;
        return $this;
    }

    public function setCreatedAt(\DateTimeInterface $datetime): \Chill\MainBundle\Doctrine\Model\TrackCreationInterface
    {
        $this->createdAt = $datetime;
        return $this;
    }

    public function setUpdatedBy(User $user): \Chill\MainBundle\Doctrine\Model\TrackUpdateInterface
    {
        $this->updatedBy = $user;
        return $this;
    }

    public function setUpdatedAt(\DateTimeInterface $datetime): \Chill\MainBundle\Doctrine\Model\TrackUpdateInterface
    {
        $this->updatedAt = $datetime;
        return $this;
    }

    /**
     * @return int|null
     */
    public function getId(): ?int
    {
        return $this->id;
    }

    /**
     * @return User|null
     */
    public function getCreatedBy(): ?User
    {
        return $this->createdBy;
    }

    /**
     * @return \DateTimeImmutable|null
     */
    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    /**
     * @return User|null
     */
    public function getUpdatedBy(): ?User
    {
        return $this->updatedBy;
    }

    /**
     * @return DateTimeImmutable|null
     */
    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    /**
     * @return DocGeneratorTemplate|null
     */
    public function getTemplate(): ?DocGeneratorTemplate
    {
        return $this->template;
    }

    /**
     * @param DocGeneratorTemplate|null $template
     * @return AccompanyingPeriodWorkEvaluationDocument
     */
    public function setTemplate(?DocGeneratorTemplate $template): AccompanyingPeriodWorkEvaluationDocument
    {
        $this->template = $template;
        return $this;
    }

}
