<?php

/*
 * Chill is a software for social workers
 *
 * Copyright (C) 2014-2021, Champs Libres Cooperative SCRLFS,
 * <http://www.champs-libres.coop>, <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\PersonBundle\Entity\AccompanyingPeriod;

use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\Collection;
use Doctrine\Common\Collections\ArrayCollection;

/**
 * ClosingMotive give an explanation why we closed the Accompanying period
 *
 * @ORM\Entity
 * @ORM\Table(name="chill_person_accompanying_period_closingmotive")
 */
class ClosingMotive
{
    /**
     * @var integer
     *
     * @ORM\Id
     * @ORM\Column(name="id", type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var array
     *
     * @ORM\Column(type="json")
     */
    private $name;
    
    /**
     * @var boolean
     *
     * @ORM\Column(type="boolean")
     */
    private $active = true;
    
    /**
     * @var self
     *
     * @ORM\ManyToOne(
     *     targetEntity="Chill\PersonBundle\Entity\AccompanyingPeriod\ClosingMotive",
     *     inversedBy="children")
     */
    private $parent = null;
    
    /**
     * Child Accompanying periods
     * @var Collection
     *
     * @ORM\OneToMany(
     *     targetEntity="Chill\PersonBundle\Entity\AccompanyingPeriod\ClosingMotive",
     *     mappedBy="parent")
     */
    private $children;
    
    /**
     * @var float
     *
     * @ORM\Column(type="float")
     */
    private $ordering = 0.0;
    
    
    /**
     * ClosingMotive constructor.
     */
    public function __construct()
    {
        $this->children = new ArrayCollection();
    }
    
    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param array $name
     *
     * @return ClosingMotive
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return array
     */
    public function getName()
    {
        return $this->name;
    }
    
    /**
     * @return bool
     */
    public function isActive(): bool
    {
        return $this->active;
    }
    
    /**
     * @param bool $active
     * @return $this
     */
    public function setActive(bool $active)
    {
        $this->active = $active;
        
        if ($this->active === FALSE) {
            foreach ($this->getChildren() as $child) {
                $child->setActive(FALSE);
            }
        }
        
        return $this;
    }
    
    /**
     * @return ClosingMotive
     */
    public function getParent()
    {
        return $this->parent;
    }
    
    /**
     * @return Collection
     */
    public function getChildren(): Collection
    {
        return $this->children;
    }
    
    /**
     * @param ClosingMotive|null $parent
     * @return ClosingMotive
     */
    public function setParent(?ClosingMotive $parent): ClosingMotive
    {
        $this->parent = $parent;
        
        if (NULL !== $parent) {
            //$parent->addChildren($this);
        }
        
        return $this;
    }
    
    /**
     * @param Collection $children
     * @return ClosingMotive
     */
    public function setChildren(Collection $children): ClosingMotive
    {
        $this->children = $children;
        
        return $this;
    }
    
    /**
     * @param ClosingMotive $child
     * @return ClosingMotive
     */
    public function addChildren(ClosingMotive $child): ClosingMotive
    {
        if ($this->children->contains($child)) {
            return $this;
        }
        
        $this->children->add($child);
        $child->setParent($this);
        
        return $this;
    }
    
    /**
     * @param ClosingMotive $child
     * @return ClosingMotive
     */
    public function removeChildren(ClosingMotive $child): ClosingMotive
    {
        if ($this->children->removeElement($child)) {
            $child->setParent(null);
        }
        
        return $this;
    }
    
    /**
     * @return float
     */
    public function getOrdering(): float
    {
        return $this->ordering;
    }
    
    /**
     * @param float $ordering
     * @return $this
     */
    public function setOrdering(float $ordering)
    {
        $this->ordering = $ordering;
        
        return $this;
    }
    
    /**
     * @return bool
     */
    public function isChild(): bool
    {
        return $this->parent !== null;
    }
    
    /**
     * @return bool
     */
    public function isParent(): bool
    {
        return $this->children->count() > 0;
    }
    
    /**
     * @return bool
     */
    public function isLeaf(): bool
    {
        return $this->children->count() === 0;
    }
    
    /**
     * @return bool
     */
    public function hasParent(): bool
    {
        return $this->parent !== null;
    }

}

