<?php

namespace Chill\PersonBundle\Entity\Person;

use Chill\MainBundle\Entity\Address;
use Chill\PersonBundle\Entity\Person;
use Doctrine\ORM\Mapping as ORM;

/**
 * Entity which associate person with his current address, through
 * household participation.
 *
 * The computation is optimized on database side.
 *
 * The validFrom and validTo properties are the intersection of
 * household membership and address validity. See @link{PersonHouseholdAddress}
 *
 * @ORM\Entity(readOnly=true)
 * @ORM\Table("view_chill_person_current_address")
 */
class PersonCurrentAddress
{
    /**
     * @ORM\Id
     * @ORM\OneToOne(targetEntity=Person::class, inversedBy="currentPersonAddress")
     * @ORM\JoinColumn(name="person_id", referencedColumnName="id")
     */
    protected Person $person;

    /**
     * @ORM\OneToOne(targetEntity=Address::class)
     */
    protected Address $address;

    /**
     * @ORM\Column(name="valid_from", type="date_immutable")
     */
    protected \DateTimeImmutable $validFrom;

    /**
     * @ORM\Column(name="valid_to", type="date_immutable")
     */
    protected ?\DateTimeImmutable $validTo;

    /**
     * @return Person
     */
    public function getPerson(): Person
    {
        return $this->person;
    }

    /**
     * @return Address
     */
    public function getAddress(): Address
    {
        return $this->address;
    }

    /**
     * This date is the intersection of household membership
     * and address validity
     *
     * @return \DateTimeImmutable
     */
    public function getValidFrom(): \DateTimeImmutable
    {
        return $this->validFrom;
    }

    /**
     * This date is the intersection of household membership
     * and address validity
     *
     * @return \DateTimeImmutable|null
     */
    public function getValidTo(): ?\DateTimeImmutable
    {
        return $this->validTo;
    }
}
