<?php

namespace Chill\PersonBundle\Entity\Relationships;

use Chill\MainBundle\Doctrine\Model\TrackCreationInterface;
use Chill\MainBundle\Doctrine\Model\TrackUpdateInterface;
use Chill\MainBundle\Entity\User;
use Chill\PersonBundle\Entity\Person;
use Chill\PersonBundle\Entity\Relationships\Relation;
use DateTimeImmutable;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Serializer\Annotation\DiscriminatorMap;
use Doctrine\ORM\Mapping\DiscriminatorColumn;
use Symfony\Component\Serializer\Annotation as Serializer;

/**
 * @ORM\Entity()
 * @ORM\Table(name="chill_person_relationships")
 * @DiscriminatorColumn(name="relation_id", type="integer")
 * @DiscriminatorMap(typeProperty="type", mapping={
 *  "relationship"=Relationship::class
 * })
 * 
 */
class Relationship implements TrackCreationInterface, TrackUpdateInterface
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     * @Serializer\Groups({"read"})
     */
    private ?int $id = null;

    /**
     * @ORM\ManyToOne(targetEntity=Person::class)
     * @ORM\JoinColumn(nullable=false)
     * @Assert\NotNull()
     * @Serializer\Groups({"read", "write"})
     */
    private ?Person $fromPerson = null;

    /**
     * @ORM\ManyToOne(targetEntity=Person::class)
     * @ORM\JoinColumn(nullable=false)
     * @Assert\NotNull()
     * @Serializer\Groups({"read", "write"})
     */
    private ?Person $toPerson = null;
    
    /**
     * @ORM\ManyToOne(targetEntity=Relation::class)
     * @ORM\JoinColumn(nullable=false, name="relation_id", referencedColumnName="id")
     * @Assert\NotNull()
     * @Serializer\Groups({"read", "write"})
     */
    private ?Relation $relation = null;
    
    /**
     * @ORM\Column(type="boolean")
     * @Assert\Type(
     *  type="bool",
     *  message="This must be of type boolean"
     * )
     * @Serializer\Groups({"read", "write"})
     */
    private bool $reverse;

    /**
     * @ORM\ManyToOne(targetEntity=User::class)
     * @ORM\JoinColumn(nullable=false)
     */
    private ?User $createdBy = null;

    /**
     * @ORM\Column(type="datetime_immutable")
     */
    private ?DateTimeImmutable $createdAt = null;

    /**
     * @ORM\ManyToOne(targetEntity=User::class)
     */
    private ?User $updatedBy = null;

    /**
     * @ORM\Column(type="datetime_immutable", nullable=true)
     */
    private ?DateTimeImmutable $updatedAt = null;


    public function getId(): ?int
    {
        return $this->id;
    }

    public function getFromPerson(): ?Person
    {
        return $this->fromPerson;
    }

    public function setFromPerson(?Person $fromPerson): self
    {
        $this->fromPerson = $fromPerson;

        return $this;
    }

    public function getToPerson(): ?Person
    {
        return $this->toPerson;
    }

    public function setToPerson(?Person $toPerson): self
    {
        $this->toPerson = $toPerson;

        return $this;
    }

    public function getReverse(): ?bool
    {
        return $this->reverse;
    }

    public function setReverse(bool $reverse): self
    {
        $this->reverse = $reverse;

        return $this;
    }

    public function getCreatedBy(): ?User
    {
        return $this->createdBy;
    }

    public function setCreatedBy(?User $user): self
    {
        $this->createdBy = $user;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeImmutable
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTimeInterface $createdAt): self
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedBy(): ?User
    {
        return $this->updatedBy;
    }

    public function setUpdatedBy(?User $updatedBy): self
    {
        $this->updatedBy = $updatedBy;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeImmutable
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(?\DateTimeInterface $updatedAt): self
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    public function getRelation(): ?Relation
    {
        return $this->relation;
    }

    public function setRelation(?Relation $relation): self
    {
        $this->relation = $relation;

        return $this;
    }
}