<?php

namespace Chill\PersonBundle\Entity\SocialWork;

use Chill\PersonBundle\Entity\AccompanyingPeriod\AccompanyingPeriodWork;
use Chill\PersonBundle\Entity\AccompanyingPeriod\AccompanyingPeriodWorkGoal;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Serializer\Annotation as Serializer;

/**
 * @ORM\Entity
 * @ORM\Table(name="chill_person_social_work_result")
 * @Serializer\DiscriminatorMap(
 *      typeProperty="type",
 *      mapping={
 *          "social_work_result":Result::class
 *      }
 *  )
 */
class Result
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     * @Serializer\Groups({"read"})
     */
    private $id;

    /**
     * @ORM\Column(type="json")
     * @Serializer\Groups({"read"})
     */
    private $title = [];

    /**
     * @ORM\Column(type="datetime", nullable=true)
     */
    private $desactivationDate;

    /**
     * @ORM\ManyToMany(targetEntity=SocialAction::class, mappedBy="results")
     */
    private $socialActions;

    /**
     * @ORM\ManyToMany(targetEntity=Goal::class, mappedBy="results")
     */
    private $goals;

    /**
     * @ORM\ManyToMany(targetEntity=AccompanyingPeriodWork::class, mappedBy="results")
     */
    private $accompanyingPeriodWorks;

    /**
     * @ORM\ManyToMany(targetEntity=AccompanyingPeriodWorkGoal::class, mappedBy="results")
     */
    private $accompanyingPeriodWorkGoals;

    public function __construct()
    {
        $this->socialActions = new ArrayCollection();
        $this->goals = new ArrayCollection();
        $this->accompanyingPeriodWorks = new ArrayCollection();
        $this->accompanyingPeriodWorkGoals = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getTitle(): array
    {
        return $this->title;
    }

    public function setTitle(array $title): self
    {
        $this->title = $title;

        return $this;
    }

    public function getDesactivationDate(): ?\DateTimeInterface
    {
        return $this->desactivationDate;
    }

    public function setDesactivationDate(?\DateTimeInterface $desactivationDate): self
    {
        $this->desactivationDate = $desactivationDate;

        return $this;
    }

    /**
     * @return Collection|SocialAction[]
     */
    public function getSocialActions(): Collection
    {
        return $this->socialActions;
    }

    public function addSocialAction(SocialAction $socialAction): self
    {
        if (!$this->socialActions->contains($socialAction)) {
            $this->socialActions[] = $socialAction;
        }

        return $this;
    }

    public function removeSocialAction(SocialAction $socialAction): self
    {
        $this->socialActions->removeElement($socialAction);

        return $this;
    }

    /**
     * @return Collection|Goal[]
     */
    public function getGoals(): Collection
    {
        return $this->goals;
    }

    public function addGoal(Goal $goal): self
    {
        if (!$this->goals->contains($goal)) {
            $this->goals[] = $goal;
        }

        return $this;
    }

    public function removeGoal(Goal $goal): self
    {
        $this->goals->removeElement($goal);

        return $this;
    }

    /**
     * @return Collection|AccompanyingPeriodWork[]
     */
    public function getAccompanyingPeriodWorks(): Collection
    {
        return $this->accompanyingPeriodWorks;
    }

    public function addAccompanyingPeriodWork(AccompanyingPeriodWork $accompanyingPeriod): self
    {
        if (!$this->accompanyingPeriodWorks->contains($accompanyingPeriod)) {
            $this->accompanyingPeriodWorks[] = $accompanyingPeriod;
        }

        return $this;
    }

    public function removeAccompanyingPeriodWork(AccompanyingPeriodWork $accompanyingPeriod): self
    {
        $this->accompanyingPeriodWorks->removeElement($accompanyingPeriod);

        return $this;
    }

    /**
     * @return Collection|AccompanyingPeriodWorkGoal[]
     */
    public function getAccompanyingPeriodWorkGoals(): Collection
    {
        return $this->accompanyingPeriodWorkGoals;
    }

    public function addAccompanyingPeriodWorkGoal(AccompanyingPeriodWorkGoal $accompanyingPeriodWorkGoal): self
    {
        if (!$this->accompanyingPeriodWorkGoals->contains($accompanyingPeriodWorkGoal)) {
            $this->accompanyingPeriodWorkGoals[] = $accompanyingPeriodWorkGoal;
        }

        return $this;
    }

    public function removeAccompanyingPeriodWorkGoal(AccompanyingPeriodWorkGoal $accompanyingPeriodWorkGoal): self
    {
        $this->accompanyingPeriodWorkGoals->removeElement($accompanyingPeriodWorkGoal);

        return $this;
    }
}
