<?php

declare(strict_types=1);

namespace Chill\PersonBundle\Form\SocialWork;

use Chill\MainBundle\Templating\TranslatableStringHelperInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Chill\MainBundle\Form\Type\TranslatableStringFormType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Chill\PersonBundle\Entity\SocialWork\SocialIssue;

class SocialIssueType extends AbstractType
{
    protected TranslatableStringHelperInterface $translatableStringHelper;

    public function __construct(
        TranslatableStringHelperInterface $translatableStringHelper
    ) {
        $this->translatableStringHelper = $translatableStringHelper;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add('title', TranslatableStringFormType::class, [
                'label' => 'Nom'
            ])
            ->add('parent', EntityType::class, [
                'class' => SocialIssue::class,
                'required'   => false,
                'choice_label' => fn (SocialIssue $issue): ?string => $this->translatableStringHelper->localize($issue->getTitle())
            ])
            ->add('desactivationDate', DateType::class, [
                'attr'  => ['class' => 'datepicker'],
                'widget'=> 'single_text',
                'format' => 'dd-MM-yyyy',
                'required' => false,
            ]);
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefault('class', SocialIssue::class);
    }
}
