<?php

/*
 * Copyright (C) 2018 Champs-Libres <info@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\PersonBundle\Menu;

use Chill\MainBundle\Routing\LocalMenuBuilderInterface;
use Knp\Menu\MenuItem;
use Symfony\Component\Security\Core\Authorization\AuthorizationCheckerInterface;
use Chill\PersonBundle\Security\Authorization\PersonVoter;
use Symfony\Component\Translation\TranslatorInterface;

/**
 * Class SectionMenuBuilder
 *
 * @package Chill\PersonBundle\Menu
 * @author Julien Fastré <julien.fastre@champs-libres.coop>
 */
class SectionMenuBuilder implements LocalMenuBuilderInterface
{
    /**
     * @var AuthorizationCheckerInterface
     */
    protected $authorizationChecker;
    
    /**
     * @var TranslatorInterface
     */
    protected $translator;
    
    /**
     * SectionMenuBuilder constructor.
     *
     * @param AuthorizationCheckerInterface $authorizationChecker
     * @param TranslatorInterface $translator
     */
    public function __construct(AuthorizationCheckerInterface $authorizationChecker, TranslatorInterface $translator)
    {
        $this->authorizationChecker = $authorizationChecker;
        $this->translator = $translator;
    }
    
    /**
     * @param $menuId
     * @param MenuItem $menu
     * @param array $parameters
     */
    public function buildMenu($menuId, MenuItem $menu, array $parameters)
    {
        if ($this->authorizationChecker->isGranted(PersonVoter::CREATE)) {
            $menu->addChild($this->translator->trans('Add a person'), [
                    'route' => 'chill_person_new'
                ])
                ->setExtras([
                    'order' => 10,
                    'icons' => [ 'plus' ]
                ]);
        }

        $menu->addChild($this->translator->trans('Create an accompanying course'), [
                'route' => 'chill_person_accompanying_course_new'
            ])
            ->setExtras([
                'order' => 11,
                'icons' => [ 'plus' ]
            ]);
    }
    
    /**
     * @return array
     */
    public static function getMenuIds(): array
    {
        return [ 'section' ];
    }
}
