<?php

namespace Chill\PersonBundle\Repository;

use Chill\MainBundle\Security\Authorization\AuthorizationHelper;
use Chill\MainBundle\Security\Resolver\CenterResolverDispatcher;
use Chill\PersonBundle\Entity\AccompanyingPeriod;
use Chill\PersonBundle\Entity\Person;
use Symfony\Component\Security\Core\Security;

final class AccompanyingPeriodACLAwareRepository implements AccompanyingPeriodACLAwareRepositoryInterface
{
    private AccompanyingPeriodRepository $accompanyingPeriodRepository;
    private Security $security;
    private AuthorizationHelper $authorizationHelper;
    private CenterResolverDispatcher $centerResolverDispatcher;

    public function __construct(AccompanyingPeriodRepository $accompanyingPeriodRepository, Security $security, AuthorizationHelper $authorizationHelper, CenterResolverDispatcher $centerResolverDispatcher)
    {
        $this->accompanyingPeriodRepository = $accompanyingPeriodRepository;
        $this->security = $security;
        $this->authorizationHelper = $authorizationHelper;
        $this->centerResolverDispatcher = $centerResolverDispatcher;
    }

    public function findByPerson(
        Person $person,
        string $role,
        ?array $orderBy = [],
        int $limit = null,
        int $offset = null
    ): array {
        $qb = $this->accompanyingPeriodRepository->createQueryBuilder('ap');
        $scopes = $this->authorizationHelper
            ->getReachableCircles($this->security->getUser(), $role,
                $this->centerResolverDispatcher->resolveCenter($person));

        if (0 === count($scopes)) {
            return [];
        }

        $qb
            ->join('ap.participations', 'participation')
            ->where($qb->expr()->eq('participation.person', ':person'))
            ->andWhere(
                $qb->expr()->orX(
                    'ap.confidential = FALSE',
                    $qb->expr()->eq('ap.user', ':user')
                )
            )
            ->andWhere(
                $qb->expr()->orX(
                    $qb->expr()->neq('ap.step', ':draft'),
                    $qb->expr()->eq('ap.createdBy', ':creator')
                )
            )
            ->setParameter('draft', AccompanyingPeriod::STEP_DRAFT)
            ->setParameter('person', $person)
            ->setParameter('user', $this->security->getUser())
            ->setParameter('creator', $this->security->getUser())
        ;
        // add join condition for scopes
        $orx = $qb->expr()->orX(
            $qb->expr()->eq('ap.step', ':draft')
        );

        foreach ($scopes as $key => $scope) {
            $orx->add($qb->expr()->isMemberOf(':scope_'.$key, 'ap.scopes'));
            $qb->setParameter('scope_'.$key, $scope);
        }
        $qb->andWhere($orx);

        return $qb->getQuery()->getResult();
    }

}
