
import { createStore } from 'vuex';
import { datetimeToISO } from 'ChillMainAssets/chill/js/date.js';
import { findSocialActionsBySocialIssue } from 'ChillPersonAssets/vuejs/_api/SocialWorkSocialAction.js';
import { create } from 'ChillPersonAssets/vuejs/_api/AccompanyingCourseWork.js';

const debug = process.env.NODE_ENV !== 'production';

const store = createStore({
  strict: debug,
  state: {
    accompanyingCourse: window.accompanyingCourse,
    socialIssues: window.accompanyingCourse.socialIssues,
    socialIssuePicked: null,
    socialActionsReachables: [],
    socialActionPicked: null,
    personsPicked: window.accompanyingCourse.participations.filter(p => p.endDate == null)
      .map(p => p.person),
    personsReachables: window.accompanyingCourse.participations.filter(p => p.endDate == null)
      .map(p => p.person),
    startDate: new Date(),
    endDate: null,
    isLoadingSocialActions: false,
    isPostingWork: false,
    errors: [],
  },
  getters: {
    hasSocialActionPicked(state) {
      console.log(state.socialActionPicked);
      return null !== state.socialActionPicked;
    },
    hasSocialIssuePicked(state) {
      console.log(state.socialIssuePicked);
      return null !== state.socialIssuePicked;
    },
    isLoadingSocialActions(state) {
      return state.isLoadingSocialActions;
    },
    isPostingWork(state) {
      return state.isPostingWork;
    },
    buildPayloadCreate(state) {
      let payload = {
        type: 'accompanying_period_work',
        socialAction: {
          type: 'social_work_social_action',
          id: state.socialActionPicked.id
        },
        startDate: {
          datetime: datetimeToISO(state.startDate)
        },
        persons: []
      };

      for (let i in state.personsPicked) {
        payload.persons.push({
          id: state.personsPicked[i].id,
          type: 'person'
        });
      }

      if (null !== state.endDate) {
        payload.endDate = {
          datetime: datetimeToISO(state.endDate)
        };
      }

      return payload;
    },
    hasErrors(state) {
      return state.errors.length > 0;
    },
  },
  mutations: {
    setSocialActionsReachables(state, actions) {
      console.log('set social action reachables');
      console.log(actions);

      state.socialActionsReachables = actions;
    },
    setSocialAction(state, socialAction) {
      console.log('socialAction', socialAction);
      state.socialActionPicked = socialAction;
    },
    setSocialIssue(state, socialIssueId) {
      console.log('set social issue', socialIssueId);
      if (socialIssueId === null) {
        state.socialIssuePicked = null;
      } else {
        let mapped = state.socialIssues
          .find(e => e.id === socialIssueId);
        console.log('mapped', mapped);
        state.socialIssuePicked = mapped;
        console.log('social issue setted', state.socialIssuePicked);
      }
    },
    setIsLoadingSocialActions(state, s) {
      state.isLoadingSocialActions = s;
    },
    setPostingWork(state) {
      state.isPostingWork = true;
    },
    setStartDate(state, date) {
      state.startDate = date;
    },
    setEndDate(state, date) {
      state.endDate = date;
    },
    setPersonsPickedIds(state, ids) {
      console.log('persons ids', ids);
      state.personsPicked = state.personsReachables
        .filter(p => ids.includes(p.id))
    },
    addErrors(state, { errors, cancel_posting }) {
      console.log('add errors', errors);
      state.errors = errors;
      if (cancel_posting) {
        state.isPostingWork = false;
      }
    },
  },
  actions: {
    pickSocialIssue({ commit }, socialIssueId) {
      console.log('pick social issue');

      commit('setIsLoadingSocialActions', true);
      commit('setSocialAction', null);
      commit('setSocialActionsReachables', []);
      commit('setSocialIssue', null);


      findSocialActionsBySocialIssue(socialIssueId).then(
        (response) => {
          console.log(response);
          console.log(response.results);
          commit('setSocialIssue', socialIssueId);
          commit('setSocialActionsReachables', response.results);
          commit('setIsLoadingSocialActions', false);
        })
        .catch(err => {
          console.error(err);
        });
    },
    submit({ commit, getters, state }) {
      console.log('submit');
      let
        payload = getters.buildPayloadCreate,
        errors = [];

      commit('setPostingWork');

      create(state.accompanyingCourse.id, payload)
        .then( ({status, data}) => {
          console.log('created return', { status, data});
          if (status === 200) {
            console.log('created, nothing to do here any more. Bye-bye!');
            window.location.assign(`/fr/person/accompanying-period/work/${data.id}/edit`);
          } else if (status === 422) {
            console.log(data);
            for (let i in data.violations) {
              console.log(i);
              console.log(data.violations[i].title);
              errors.push(data.violations[i].title);
            }
            console.log('errors after reseponse handling', errors);
            console.log({errors, cancel_posting: true});
            commit('addErrors', { errors, cancel_posting: true });
          }
        });
    },
  },

});

export { store };
