<?php

/*
 * Copyright (C) 2015 Julien Fastré <julien.fastre@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\PersonBundle\Security\Authorization;

use Chill\MainBundle\Security\Authorization\AbstractChillVoter;
use Chill\MainBundle\Entity\User;
use Chill\MainBundle\Security\Authorization\AuthorizationHelper;
use Chill\MainBundle\Security\Authorization\VoterHelperFactoryInterface;
use Chill\MainBundle\Security\Authorization\VoterHelperInterface;
use Chill\MainBundle\Security\ProvideRoleHierarchyInterface;
use Chill\MainBundle\Security\Resolver\CenterResolverDispatcher;
use Chill\PersonBundle\Entity\Person;
use Chill\MainBundle\Entity\Center;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Role\Role;

class PersonVoter extends AbstractChillVoter implements ProvideRoleHierarchyInterface
{
    const CREATE = 'CHILL_PERSON_CREATE';
    const UPDATE = 'CHILL_PERSON_UPDATE';
    const SEE    = 'CHILL_PERSON_SEE';
    const STATS  = 'CHILL_PERSON_STATS';
    const LISTS  = 'CHILL_PERSON_LISTS';
    const DUPLICATE = 'CHILL_PERSON_DUPLICATE';

    protected VoterHelperInterface $voter;

    public function __construct(
        VoterHelperFactoryInterface $voterFactory
    ) {
        $this->voter = $voterFactory
            ->generate(self::class)
            ->addCheckFor(Center::class, [self::STATS, self::LISTS, self::DUPLICATE])
            ->addCheckFor(Person::class, [self::CREATE, self::UPDATE, self::SEE, self::DUPLICATE])
            ->addCheckFor(null, [self::CREATE] )
            ->build()
            ;
    }

    protected function supports($attribute, $subject)
    {
        return $this->voter->supports($attribute, $subject);
    }

    protected function voteOnAttribute($attribute, $subject, TokenInterface $token)
    {
        return $this->voter->voteOnAttribute($attribute, $subject, $token);
    }

    private function getAttributes()
    {
        return array(self::CREATE, self::UPDATE, self::SEE, self::STATS, self::LISTS, self::DUPLICATE);
    }

    public function getRoles()
    {
        return $this->getAttributes();
    }

    public function getRolesWithoutScope()
    {
        return $this->getAttributes();
    }

    public function getRolesWithHierarchy()
    {
        return [ 'Person' => $this->getRoles() ];
    }

}
