<?php

use PHPUnit\Framework\TestCase;
use Chill\PersonBundle\EventListener\PersonEventListener;
use Chill\PersonBundle\Entity\Person;
use Chill\PersonBundle\Entity\PersonAltName;
use Doctrine\Persistence\Event\LifecycleEventArgs;

class PersonCreateEventTest extends TestCase
{
    /**
    * @dataProvider generateNames
    */
    public function testOnPrePersist($firstname, $firstnameExpected, $lastname, $lastnameExpected)
    {
        $listener = new PersonEventListener();

        $person = new Person();

        $person->setFirstName($firstname);
        $person->setLastName($lastname);

        $listener->prePersistPerson($person);

        $this->assertEquals($firstnameExpected, $person->getFirstName());
        $this->assertEquals($lastnameExpected, $person->getLastName());
    }

    /**
    * @dataProvider generateAltNames
    */
    public function testAltNamesOnPrePersist($altname, $altnameExpected)
    {
        $listener = new PersonEventListener();

        $personAltname = new PersonAltName();

        $personAltname->setLabel($altname);

        $listener->prePersistAltName($personAltname);

        $this->assertEquals($altnameExpected, $personAltname->getLabel());
    }

    public function generateNames(): iterator
    {
        yield ['émelie-marie', 'Émelie-Marie', 'lenaerts', 'LENAERTS'];
        yield ['jean-marie', 'Jean-Marie', 'lenaerts', 'LENAERTS'];
        yield ['vinCENT', 'Vincent', 'fastré', 'FASTRÉ'];
        yield ['Vincent', 'Vincent', 'van Gogh', 'VAN GOGH'];
        yield ['André marie', 'André Marie', 'Bah', 'BAH'];
    }

    public function generateAltNames(): iterator
    {
        yield ['vinCENT', 'VINCENT'];
        yield ['jean-marie', 'JEAN-MARIE'];
        yield ['fastré', 'FASTRÉ'];
        yield ['émile', 'ÉMILE'];
    }
}
