<?php

/*
 * Copyright (C) 2015 Julien Fastré <julien.fastre@champs-libres.coop>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Chill\TaskBundle\DataFixtures\ORM;

use Doctrine\Common\DataFixtures\AbstractFixture;
use Doctrine\Common\DataFixtures\OrderedFixtureInterface;
use Doctrine\Persistence\ObjectManager;
use Chill\MainBundle\DataFixtures\ORM\LoadPermissionsGroup;
use Chill\MainBundle\Entity\RoleScope;
use Chill\MainBundle\DataFixtures\ORM\LoadScopes;
use Chill\TaskBundle\Security\Authorization\TaskVoter;

/**
 * Add a role UPDATE & CREATE for all groups except administrative,
 * and a role SEE for administrative
 *
 * @author Julien Fastré <julien.fastre@champs-libres.coop>
 */
class LoadTaskACL extends AbstractFixture implements OrderedFixtureInterface
{
    public function getOrder()
    {
        return 16000;
    }


    public function load(ObjectManager $manager)
    {
        foreach (LoadPermissionsGroup::$refs as $permissionsGroupRef) {
            $permissionsGroup = $this->getReference($permissionsGroupRef);
            foreach (LoadScopes::$references as $scopeRef){
                $scope = $this->getReference($scopeRef);
                //create permission group
                switch ($permissionsGroup->getName()) {
                    case 'social':
                        if ($scope->getName()['en'] === 'administrative') {
                            break 2; // we do not want any power on administrative
                        }
                        break;
                    case 'administrative':
                    case 'direction':
                        if (in_array($scope->getName()['en'], array('administrative', 'social'))) {
                            break 2; // we do not want any power on social or administrative
                        }
                        break;
                }

                printf("Adding CHILL_TASK_TASK_UPDATE & CHILL_TASK_TASK_CREATE & Chill_TASK_TASK_DELETE permissions to %s "
                        . "permission group, scope '%s' \n",
                        $permissionsGroup->getName(), $scope->getName()['en']);
                $roleScopeUpdate = (new RoleScope())
                            ->setRole(TaskVoter::UPDATE)
                            ->setScope($scope);
                $permissionsGroup->addRoleScope($roleScopeUpdate);
                $roleScopeCreateP = (new RoleScope())
                            ->setRole(TaskVoter::CREATE_PERSON)
                            ->setScope($scope);
                $permissionsGroup->addRoleScope($roleScopeCreateP);
                $roleScopeCreateC = (new RoleScope())
                    ->setRole(TaskVoter::CREATE_COURSE)
                    ->setScope($scope);
                $permissionsGroup->addRoleScope($roleScopeCreateC);
                $roleScopeDelete = (new RoleScope())
                            ->setRole(TaskVoter::DELETE)
                            ->setScope($scope);
                $permissionsGroup->addRoleScope($roleScopeDelete);

                $manager->persist($roleScopeUpdate);
                $manager->persist($roleScopeCreateP);
                $manager->persist($roleScopeCreateC);
                $manager->persist($roleScopeDelete);
            }

        }

        $manager->flush();
    }

}
