<?php

declare(strict_types=1);

namespace Chill\TaskBundle\Form;

use Chill\MainBundle\Security\Resolver\CenterResolverDispatcher;
use Chill\MainBundle\Security\Resolver\ScopeResolverDispatcher;
use Chill\TaskBundle\Security\Authorization\TaskVoter;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Chill\MainBundle\Form\Type\ChillDateType;
use Chill\MainBundle\Entity\Center;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Chill\MainBundle\Form\Type\UserPickerType;
use Chill\MainBundle\Form\Type\ScopePickerType;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Security\Core\Role\Role;
use Chill\MainBundle\Form\Type\DateIntervalType;
use Chill\MainBundle\Form\Type\ChillTextareaType;

class SingleTaskType extends AbstractType
{
    private ParameterBagInterface $parameterBag;
    private CenterResolverDispatcher $centerResolverDispatcher;
    private ScopeResolverDispatcher $scopeResolverDispatcher;

    public function __construct(ParameterBagInterface $parameterBag, CenterResolverDispatcher $centerResolverDispatcher, ScopeResolverDispatcher $scopeResolverDispatcher)
    {
        $this->parameterBag = $parameterBag;
        $this->centerResolverDispatcher = $centerResolverDispatcher;
        $this->scopeResolverDispatcher = $scopeResolverDispatcher;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $center = null;
        $isScopeConcerned = false;

        if (NULL !== $task = $options['data']) {
            $center = $this->centerResolverDispatcher->resolveCenter($task);
            $isScopeConcerned = $this->scopeResolverDispatcher->isConcerned($task);
        }

        $builder
            ->add('title', TextType::class)
            ->add('description', ChillTextareaType::class, [
                'required' => false
            ])
            ->add('assignee', UserPickerType::class, [
                'required' => false,
                'center' => $center,
                'role'   => TaskVoter::SHOW,
                'placeholder' => 'Not assigned',
                'attr' => [ 'class' => ' select2 ']
                ])
            ->add('startDate', ChillDateType::class, [
                'required' => false
            ])
            ->add('endDate', ChillDateType::class, [
                'required' => false
            ])
            ->add('warningInterval', DateIntervalType::class, [
                'required' => false
            ]);

        if ($isScopeConcerned && $this->parameterBag->get('chill_main')['acl']['form_show_scopes']) {
            $builder
                ->add('circle', ScopePickerType::class, [
                    'center' => $center,
                    'role' => $options['role'],
                    'required' => false
                ]);
        }
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver
            ->setRequired('role')
            ->setAllowedTypes('role', [ Role::class, 'string' ])
            ;
    }
}
