<?php

namespace Chill\ThirdPartyBundle\Repository;

use Chill\MainBundle\Security\Authorization\AuthorizationHelper;
use Chill\ThirdPartyBundle\Entity\ThirdParty;
use Doctrine\ORM\QueryBuilder;
use Symfony\Component\Security\Core\Security;

final class ThirdPartyACLAwareRepository implements ThirdPartyACLAwareRepositoryInterface
{
    private Security $security;
    private AuthorizationHelper $authorizationHelper;
    private ThirdPartyRepository $thirdPartyRepository;

    public function __construct(Security $security, AuthorizationHelper $authorizationHelper, ThirdPartyRepository $thirdPartyRepository)
    {
        $this->security = $security;
        $this->authorizationHelper = $authorizationHelper;
        $this->thirdPartyRepository = $thirdPartyRepository;
    }

    public function listThirdParties(
       string $role,
       ?string $filterString,
       ?array $orderBy = [],
       ?int $limit = null,
       ?int $offset = null
    ): array {
        $qb = $this->buildQuery($filterString);

        foreach ($orderBy as $sort => $direction) {
            $qb->addOrderBy('tp.'.$sort, $direction);
        }

        $qb->setFirstResult($offset)
            ->setMaxResults($limit);

        return $qb->getQuery()->getResult();
    }

    public function countThirdParties(
        string $role,
        ?string $filterString
    ): int {
        $qb = $this->buildQuery($filterString);
        $qb->select('count(tp)');

        return $qb->getQuery()->getSingleScalarResult();
    }

    public function buildQuery(?string $filterString = null): QueryBuilder
    {
        $qb = $this->thirdPartyRepository->createQueryBuilder('tp');

        if (NULL !== $filterString) {
            $qb->andWhere($qb->expr()->like('tp.canonicalized', 'LOWER(UNACCENT(:filterString))'))
                ->setParameter('filterString', '%'.$filterString.'%');
        }

        return $qb;
    }
}
